#!/usr/bin/python
import subprocess
import string
import os
import os.path
import sys
import ConfigParser

abs_path = os.path.abspath(sys.argv[0])
basename = os.path.basename(sys.argv[0])
dirname = abs_path.replace(basename, "")
if not "%s/diskmon/lib" % dirname in sys.path:
    sys.path.insert(0,"%sdiskmon/lib" % dirname) # always with regard to parent script
if not "/usr/bp/lib/monitoring" in sys.path:
    sys.path.append("/usr/bp/lib/monitoring")
    
from bpdb     import *   # include config << bpdb as well
from optparse import OptionParser

parser = OptionParser()
parser.add_option("-d", "--debug",   dest="debug", default=False, action='store_true',
                  help="debug option to show detail commands to test")
parser.add_option("-v", "--verbose", dest="verbose", default=False, action='store_true',
                  help="verbose option to show output for debug purpose")
parser.add_option("-f", "--force", dest="force", default=False, action='store_true',
                  help="sending trap ignoring pass threshold (Sunday midnight)")
parser.add_option("-a", "--ana",   dest="ana", default=False, action='store_true',
                  help="run all the ana functions")
parser.add_option("--anaProtected",   dest="anaProtected", default=False, action='store_true',
                  help="analysis to protected number of agents/applications and total amount of files over file level/VMESX/HyperV/Exchage/MSSQL/Sharepoint/Oracle")
parser.add_option("--anaEsxServers",  dest="anaEsxServers", default=False, action='store_true',
                  help="analysis to get ESX server version information")
parser.add_option("--anaCloudHook",   dest="anaCloudHook", default=False, action='store_true',
                  help="analysis to get cloudhook information")
parser.add_option("-c", "--confirm",   dest="confirm", default=False, action='store_true',
                  help="Without confirm flag, snmp won't be sent")
(opts, args) = parser.parse_args()

CONFIG.debug        =opts.debug
CONFIG.verbose      =opts.verbose
CONFIG.confirm      =opts.confirm
CONFIG.anaProtected =opts.anaProtected
CONFIG.anaEsxServers=opts.anaEsxServers
CONFIG.anaCloudHook =opts.anaCloudHook
CONFIG.force        =opts.force

if opts.ana:
    CONFIG.anaProtected =True
    CONFIG.anaEsxServers=True
    CONFIG.anaCloudHook=True

CONFIG.nAsset =0  # Number of assets in node table:
                  # if they are, then they should have "System Metadata" as an application_instance.
                  # Also, associated systems (replication source) can have it 
    
from trap     import *   # include config << bpdb as well
    
        
def main():
    """
    anaProtected:
       * analysis to protected number of agents/applications and total amount of files over file level/VMESX/HyperV/Exchage/MSSQL/Sharepoint/Oracle
       * To get total amount of backups, we sum over the total_megs over the last master/full 
       * output: [INFO] Status of protection [file-level](60,60,55,43,11087G) [SQL Server](19,10,0,1,0G) [Hyper-V](34,22,12,0,209G) [UCS Service Profile](0,0,0,0,0G) [Oracle](1,1,0,0,0G) [Exchange](1,1,1,0,2G) [VMware](311,107,7,0,183G) [SharePoint](0,0,0,0,0G)
       * Each parenthesis consists of 5 numbers:
         (1) number of instances found from bp.application_instances
         (2) number of current instances found from bp.application_instances
         (3) number of instances which have full/master backup
         (4) number of instances synced (in replication)
         (5) sum of last full/master backups with GB unit

       OSCensus: (8.1.0)
       * Due to limitation of trap length allowed, compacting message has been attempted.
       * One pairs of two digit hexadecimal followed by indefinite number of hex digit for counting needed:
       *    (1) OS  type defined in include/bp/nodetype.h
       *    (2) Count
       (8.2.0) : -added ff00 where ff means number of DPU which has system metadata
                 -Correctly deduct number of DPUs from linux os count
                 
       DedupNT: (8.2.0)
       * Due to limitation of trap length allowed, compacting message has been attempted.
       * bitmap +=2^0 if MetadataDirectory is assigned
       * bitmap +=2^1 if synchronizeUpdates being No (MetadataDirectory should exist)
       * bitmap +=2^2 if enableInlineDedup being Yes
       * bitmap +=2^3 if there is any application instances having inlinededup ON (1)

       Output: [INFO] Status of protection [VMware](0,0,0,0,0G) [NDMP Device](0,0,0,0,0G) [Oracle](0,0,0,0,0G) [SharePoint](0,0,0,0,0G) [UCS Service Profile](0,0,0,0,0G) [SQL Server](0,0,0,0,0G) [Hyper-V](0,0,0,0,0G) [file-level](252,252,1,250,20G) [Exchange](0,0,0,0,0G)  [INFO] OS_Census 14fa|2a1* [INFO] DedupNT f
       
    anaEsxServers
       * analysis to count ESX version
       * output: [INFO] Esx Servers 6 current among 8 found 3.5.0(0,0) 4.0.0(0,0) 4.1.0(0,0) 5.0.0(4,3) 5.1.0(3,3) 5.5.0(1,0) 6.0.0(0,0)
       * Each parenthesis consists of 2 numbers:
         (1) number of esx server found from bp.vmware_esx_servers
         (2) number of current esx server found from bp.vmware_esx_servers

       ApplicationVersion: (8.2.0)
       * Due to limitation of trap length allowed, compacting message has been attempted.
       * Two pairs of two digit hexadecimal followed by indefinite number of hex digit for counting needed for each application_instance:
       *    (1) OS  type defined in include/bp/nodetype.h
       *    (2) App type defined in bp.application_lookup
       *    (3) Count

       SocketCounts: (8.2.0)
       * Want to know Socket distribution over protected hypervisor
       * VMware|HyperV|XenServer|Else (physical agent)
       
       Output: [INFO] Esx Servers 34 current among 41 found 3.5.0(0,0) 4.0.0(0,0) 4.1.0(0,0) 5.0.0(0,0) 5.1.0(2,2) 5.5.0(39,32) 6.0.0(0,0)  [INFO] Application_Version_Census 14011|142814e|24013|24044|27011|271826|0b012|24169|0b644*
       Output: [INFO] Esx Servers 34 current among 41 found 3.5.0(0,0) 4.0.0(0,0) 4.1.0(0,0) 5.0.0(0,0) 5.1.0(2,2) 5.5.0(39,32) 6.0.0(0,0)  [INFO] AppVer 14011|142814e|24013|24044|27011|271826|0b012|24169|0b644* [INFO] Sockets 0|2|0|28*
         
    anaCloudHook
       * anaysis to gather cloudhook information from
          # /usr/bp/bin/cloudadm status
          cloudadm version 1.08
          CLOUD TYPE      | STORAGE       | ONLINE | MAX_GB       | USED_GB
          cloud:s3        | s3plain       |   t    | 250          | 224
          cloud:swift     | rack1         |   t    | 520          | 19
          cloud:swift     | rack_test     |   t    | 0            | 0
          cloud:gs        | gsfolder      |   f    | 520          | 0
       * output:  [INFO] Cloudhook information [cloud:s3](2, 1, 250G, 209G) [cloud:swift](2, 2, 520G, 12G) [cloud:gs](1,0,520G,19G)
       * Each parenthesis consists of 3 numbers:
         (1) Number of Storages Found
         (2) Number of Active Storages
         (3) Max GB allowed (GB)
         (4) Total Used Space (GB)
       """


    message = ''
    category_id = 1 # platform
    severity=3 #informational
    isopen = 0
    doSendInfoTrap=True
    if CONFIG.anaEsxServers:
        message =getAnaEsxServers()
        if len(message):    
            shortName="AnaESXservers"
            trapToSend = Trap(category_id, shortName, message, severity, isopen, 150) # This is for version information trap
            print trapToSend.message
            if does_pass_threshold(shortName,6) or CONFIG.force:  #Crontab configured to send every 7 days; 6 days is set just to prevent edge case
                trapToSend.send_trap(doSendInfoTrap)

    if CONFIG.anaProtected:
        message =getAnaProtected()
        if len(message):    
            shortName="AnaProtected"
            trapToSend = Trap(category_id, shortName, message, severity, isopen, 150)
            print trapToSend.message
            if does_pass_threshold(shortName,6) or CONFIG.force:  #Crontab configured to send every 7 days; 6 days is set just to prevent edge case
                trapToSend.send_trap(doSendInfoTrap) #Default is not sending a trap if it's informational trap


    if CONFIG.anaCloudHook:
        message =getAnaCloudHook()
        if len(message):    
            shortName="AnaCloudHook"
            trapToSend = Trap(category_id, shortName, message, severity, isopen, 150) # This is for version information trap
            print trapToSend.message
            if does_pass_threshold(shortName,6) or CONFIG.force:  #Crontab configured to send every 7 days; 6 days is set just to prevent edge case
                trapToSend.send_trap(doSendInfoTrap)
                
    return


def getAnaCloudHook():
    enum = {'cloud:s3':0, 'cloud:swift':1, 'cloud:gs':2}
    cnts = []
    cnts_current = []
    max_gbs =[]
    used_gbs = []
    n_found =0
    for i in range(len(enum)):  # initialize
        cnts.append(0)
        cnts_current.append(0)
        max_gbs.append(0)
        used_gbs.append(0)
        
    for line in get_printout("/usr/bp/bin/cloudadm status 2>&1| /bin/grep cloud:"):
        # gathering data
        words = line.split("|")
        vendor = words[0].strip()
        online = True
        if words[2].strip() == "f":
            online = False
        max_gb  = int(words[3].strip())    
        used_gb = int(words[4].strip())    

        # measuring
        n_found +=1
        cnts[enum[vendor]] +=1
        if online:
            cnts_current[enum[vendor]] +=1
        max_gbs[enum[vendor]]  += max_gb
        used_gbs[enum[vendor]] += used_gb

    if not n_found:
        return ''

    message = "[INFO] Cloudhook information "
    for i in range(len(enum)):
        theKey =''
        for key,value in enum.items():
            if i==value: theKey = key
        message += "[%s](%d,%d,%dG,%dG) " %( theKey, cnts[i], cnts_current[i], max_gbs[i], used_gbs[i])

    return message    
    

def getAnaEsxServers():
    ####
    vmware_esx_servers = get_db_entries("vmware_esx_servers")
    enum ={'3.5.0':0, '4.0.0':1, '4.1.0':2, '5.0.0':3, '5.1.0':4, '5.5.0':5, '6.0.0':6}
    enum_hv ={'VMware':0, 'HyperV':1, 'XenServer':2, 'Else':3} 
    cnts, cnts_current, total_num_cpu_sockets =[],[],[]
    n_current, n_found=0,0
    
    for i in range(len(enum)):
        cnts.append(0)
        cnts_current.append(0)

    for i in range(len(enum_hv)):
        total_num_cpu_sockets.append(0)

    for server in vmware_esx_servers:
        cnts[enum[server.version]] += 1
        n_found +=1
        if server.current:
            total_num_cpu_sockets[enum_hv['VMware']]+=server.num_cpu_sockets
            n_current +=1
            cnts_current[enum[server.version]] += 1

    ###### nodes join application_instances        For OS
    psql_command  ="/usr/bin/psql bpdb postgres -tq -c \"SELECT os,app_id,instance_id,n.node_no "
    psql_command +="FROM bp.nodes as n INNER JOIN bp.application_instances as a ON n.node_no=a.node_no "
    psql_command +="ORDER BY instance_id\" | grep -v ^$"       
    instances = commands.getoutput(psql_command).splitlines()

    enum_app,tmpCnt,keyholder ={},0,[]
    total_instances=0
    ## Need 2 iteration of loops.
    #  1st time: fill up enum_app
    #  2nd time: count them based on enum app
    for i in range(len(instances)):
        line=instances[i]
        words=line.split("|")
        os=int(words[0])
        app_id=int(words[1])

        first_hex=hex(os)[2:]
        if len(first_hex)==1: first_hex="0"+first_hex
        second_hex=hex(app_id)[2:]
        if len(second_hex)==1: second_hex="0"+second_hex

        key="%s%s"% (first_hex, second_hex)
        total_instances+=1
        if not key in keyholder:
            keyholder.append(key)
            enum_app[key]=tmpCnt
            tmpCnt+=1

    message ="[INFO] Esx Servers %d current among %d found " %(n_current,len(vmware_esx_servers))
    for i in range(len(enum)):
        theKey =''
        for key,value in enum.items():
            if i==value: theKey = key
        message += "%s(%d,%d) " %( theKey, cnts[i], cnts_current[i])

    ####    
    cnts=[]        
    for i in range(len(enum_app)):
        cnts.append(0)

    for i in range(len(instances)):
        line=instances[i]
        words=line.split("|")
        os=int(words[0])
        app_id=int(words[1])
    
        first_hex=hex(os)[2:]
        if len(first_hex)==1: first_hex="0"+first_hex
        second_hex=hex(app_id)[2:]
        if len(second_hex)==1: second_hex="0"+second_hex

        key="%s%s"% (first_hex, second_hex)
        cnts[enum_app[key]] += 1

    
    ###### application_instances join application_lookup to figure out HyperV or NOT
    psql_command  ="/usr/bin/psql bpdb postgres -tq -c \"SELECT node_no,instance_id,ai.app_id,name "
    psql_command +="FROM bp.application_instances AS ai,bp.application_lookup AS al "
    psql_command +="WHERE ai.app_id=al.app_id and name NOT LIKE '%VMware%' ORDER BY node_no\" | grep -v ^$"
    appinstances =commands.getoutput(psql_command).splitlines()

    psql_command  ="/usr/bin/psql bpdb postgres -tq -c \"SELECT node_no,os,version,num_cpu_sockets "
    psql_command +="FROM bp.nodes "
    psql_command +="ORDER BY node_no\" | grep -v ^$"
    nodes =commands.getoutput(psql_command).splitlines()

    nAsset=0
    for node in nodes:
        values=node.split("|")
        node_no=int(values[0])
        num_cpu_socket=int(values[3])
        hasHVapp=False
        hasSystemMetadata=False
        for app in appinstances:
            appvalues=app.split("|")
            if node_no != int(appvalues[0]): continue
            if "System Metadata" in appvalues[3]:
                hasSystemMetadata=True
            elif "Hyper-V" in appvalues[3]:
                hasHVapp=True
                break
        if hasSystemMetadata: 
            nAsset+=1
            continue
        if hasHVapp:
            total_num_cpu_sockets[enum_hv['HyperV']]+=num_cpu_socket
        else:    
            total_num_cpu_sockets[enum_hv['Else']]+=num_cpu_socket
        
    CONFIG.nAsset=nAsset

    resString=''
    for i in range(len(enum_hv)):
        theKey=''
        for key,value in enum_hv.items():
            if value==i: theKey=key
        theHex=hex(total_num_cpu_sockets[enum_hv[theKey]])[2:]
        #print theKey, total_num_cpu_sockets[enum_hv[theKey]]

        #nHexDigit=3
        #if theKey=='Else': nHexDigit=5
        #for i in range(nHexDigit-len(theHex)):
        #    theHex="0"+theHex
        resString+=theHex+"|"
    resString=resString[:-1]
    
    # Moved due to the need to get nAsset        
    cnts[enum_app['1401']]-=nAsset
    #message += " [INFO] Application_Version_Census "    
    message += " [INFO] AppVer "    
    for i in range(len(enum_app)):
        theKey =''
        for key,value in enum_app.items():
            if i==value: theKey = key

        if theKey=="141e" or theKey=="143c": continue # archive and SystemMetaData
        if cnts[enum_app[theKey]]==0: continue
        
        cnt_hex=hex(cnts[enum_app[theKey]])[2:]
        if CONFIG.debug: print "%s %4d %s" %(theKey,cnts[enum_app[theKey]],cnt_hex)
        if i!=0: message+="|"
        message += "%s%s"%(theKey,cnt_hex)
        
    message+="*"

    message += " [INFO] Sockets %s*" % resString    
#        if CONFIG.debug:
#            print "%2d, %3d, %s, %s"% (os,count,format(os,'02x'),format(count,'02x'))
#
#        if i!=0: message+="|"
#        message+=format(os,'02x')+format(count,'02x')
        

    return message

def getAnaProtected():
    application_instances = get_db_entries("application_instances")
    backup_reps = get_db_entries("backup_reps") # collecting backups of which properties > 0
    for bckup in backup_reps:
        bckup.fillAttribute(application_instances) # pointer
        
    protected_instances = get_protected_instances(backup_reps)
    
    now = int(time.time())
    sums=[]
    cnts_found=[]
    cnts_currnt=[]
    cnts_proted=[]
    cnts_synced=[]
    import commands      
    psql_command = "/usr/bin/psql bpdb postgres -tq -c \"SELECT type FROM (SELECT * FROM bp.application_lookup ORDER BY app_id) AS a GROUP BY type\" | grep -v ^$"       
    if CONFIG.debug:
        print "#%s:getAnaProtected()" % basename
        print "     ",  psql_command
    app_types = commands.getoutput(psql_command).splitlines()

    enum= {}
    tmpCnt =0
    for i  in range(len(app_types)):
        app_type = app_types[i].strip()
        if app_type == "System Metadata" or app_type=="Archive": continue
        enum[app_type] = tmpCnt
        tmpCnt +=1

    for i in range(len(enum)):
        sums.append(0)
        cnts_found.append(0)
        cnts_currnt.append(0)
        cnts_proted.append(0)
        cnts_synced.append(0)


    application_lookup = get_db_entries("application_lookup")
    for app in application_instances:
        app_id = app.app_id
        app_type=get_application_type(application_lookup, app_id)
        if app_type == "System Metadata" or app_type=="Archive": continue
        cnts_found[enum[app_type]]+=1
        if app.sync:               cnts_synced[enum[app_type]]+=1
        if app.current:            cnts_currnt[enum[app_type]]+=1

    # loop over instances 
    for anInstance in protected_instances:
        app_id = anInstance[3]
        first_start_time = 0
        start_time = 0
        total_megs = 0
        node_type =-9
        os = ''
        type = ''
        # loop over backups including Master or Full which is the latest only
        for b in backup_reps:
            if not "Master" in b.backup_type and not "FULL" in b.backup_type: continue
            #if not b.sync: continue
            if anInstance[0]==b.node_no and anInstance[2]==b.instance_id:
                if start_time < b.start_time:
                    if start_time == 0:
                        first_start_time = b.start_time
                        node_type = b.node_type
                        os = b.os
                        type = b.backup_type
                    start_time = b.start_time
                    total_megs = b.total_megs
        app_type=get_application_type(application_lookup, app_id)
        if app_type == "System Metadata" or app_type=="Archive": continue
        cnts_proted[enum[app_type]]+=1
        sums[enum[app_type]]+=total_megs


    for key,value in enum.items():
        sums[value] = int(sums[value]/1024.+0.5)

    message = "[INFO] Status of protection "    
    for i in range(len(enum)):
        theKey =''
        for key,value in enum.items():
            if i==value: theKey = key
        message += "[%s](%d,%d,%d,%d,%dG) " \
                   %(theKey,cnts_found[enum[theKey]],cnts_currnt[enum[theKey]],\
                     cnts_proted[enum[theKey]],cnts_synced[enum[theKey]],sums[enum[theKey]])


    # OS CENSUS    
    import commands      
    psql_command = "/usr/bin/psql bpdb postgres -tq -c \"SELECT os,count(os) FROM bp.nodes GROUP BY os ORDER BY os\" | grep -v ^$"       
    app_types = commands.getoutput(psql_command).splitlines()
    if CONFIG.debug:
        app_types=[]
        app_types.append(" 20 |    12")
        app_types.append(" 35 |     3")
        app_types.append(" 36 |    10")
        app_types.append(" 37 |    10")
        app_types.append(" 38 |    10")
        app_types.append(" 39 |    10")
        app_types.append(" 40 |    10")
        app_types.append(" 41 |     2")
        app_types.append(" 42 |     3")
            
    # How to convert hexadecimal to decimal: int('2a',16)
    message +=' [INFO] OS_Census '    
    for i in range(len(app_types)):
        line =app_types[i]
        words=line.split("|")
        os=int(words[0])
        count=int(words[1])
        if os==20: count-=CONFIG.nAsset   # deducting unitrends asset such as DPU or replication source
        if count==0: continue
        
        if i!=0: message+="|"
        #message+=format(os,'02x')+format(count,'02x')  # new in 2.6
        first_hex=hex(os)[2:]
        if len(first_hex)==1: first_hex="0"+first_hex
        second_hex=hex(count)[2:]
        #if len(second_hex)==1: second_hex="0"+second_hex

        if CONFIG.debug:
            print "%2d, %3d, %s, %s"% (os,count,first_hex,second_hex)

        message+=first_hex + second_hex

    message+="|ff%s*" %hex(CONFIG.nAsset)[2:]    


    # DedupNT
    bitmap =0
    if get_first_word("/usr/bp/bin/bputil -g fileDedup MetadataDirectory ERROR /usr/bp/bpinit/master.ini") != "ERROR":
        bitmap +=1
        if get_first_word("/usr/bp/bin/bputil -g fileDedup synchronizeUpdates ERROR /usr/bp/bpinit/master.ini").lower() == "no":
            bitmap +=2
    if get_first_word("/usr/bp/bin/bputil -g fileDedup enableInlineDedup ERROR /usr/bp/bpinit/master.ini").lower() == "yes":
        bitmap +=4
    
    nAgent_InlineDedup =0
    if int(get_first_word("/usr/bin/psql bpdb postgres -c \"select * from bp.application_instances limit 1\" | grep inline_dedup | wc -l")):
        nAgent_InlineDedup =\
		get_first_word("/usr/bin/psql bpdb postgres -tq -c \"select count(*) from bp.application_instances where inline_dedup!=0\" | grep -v ^$")
    if int(nAgent_InlineDedup): bitmap +=8
    message +=' [INFO] DedupNT %s' %hex(bitmap)[2:]


    return message


def get_application_type(application_lookup, app_id):
    for app in application_lookup:
        if app_id==app.app_id:
            return app.type
    return "NA"

#head -10 /usr/bp/logs.dir/vcd-worker-188.log | grep Hashing:137
def get_protected_instances(backup_reps):
    protected_instances=[]
    for b in backup_reps:
        if b.instance_id<=0 or b.app_id<0: continue
        aTuple = (b.node_no, b.node_name, b.instance_id, b.app_id, b.os, b.osid)
        found = False
        for node in protected_instances:
            if aTuple[0]==node[0] and aTuple[2]==node[2]:
                found = True
                break
        if not found: protected_instances.append( aTuple )
    return protected_instances    
    
def does_pass_threshold(shortName,days):
    """
    checks if elapsed days from the last time sending traps is longer than threshold (days)

    Keyword Arguments
    string shortName  : corresponds to 'object' in bp.snmp_history table to check the last time sending trap with the object
    int    days       : minimum days (threshold) to wait for sending the next one

    Returns:
    returns True if it is older than days
    """

    if not CONFIG.dbON:  # don't want to send a trap if db is down for analysis trap...
        return False
       #if CONFIG.dbON and os.path.exists("/var/log/monitoring_disks/bp.snmp_history.dbout"):
       #   command = "tac /var/log/monitoring_disks/bp.snmp_history.dbout | grep %s" % shortName 
    identifier =shortName
    psql_command ="/usr/bin/psql bpdb postgres -tq -c \""
    psql_command+="SELECT id,oid,time_sent,destination,community,traptype,regexp_replace(object,E'[\\n\\r]+',' ','g'),severity,status,description "
    psql_command+="FROM bp.snmp_history WHERE object LIKE '%%%s' ORDER BY id DESC LIMIT 1\" | grep -v ^$" % (identifier)
    import commands      
    import time, datetime

    if CONFIG.debug:
        print "#%s:does_pass_threshold()" % basename
        print "     ",  psql_command

    lines = commands.getoutput(psql_command).splitlines()
    if len(lines):
        words = lines[0].split("|")
        time_sent =get_time_stamp(words[2].strip())
        offset = days*24
        threshold = time_sent + datetime.timedelta(hours=offset)
        now = datetime.datetime.now()
        if now>threshold:
            return True
        else:
            return False
          
    return True  # if no entries found, then we have to send a trap

def get_time_stamp(stat_time):
    import datetime
    words = stat_time.split()
    #-rw-rw-r-- 1 root loggers     622 2013-07-30 13:29:12.324820223 -0400 /usr/bp/logs.dir/vcd-worker-5.log
    dateSegs = words[0].split("-")
    timeSegs = words[1].split(":")
    timestamp_start = datetime.datetime(int(dateSegs[0]), int(dateSegs[1]), int(dateSegs[2]), int(timeSegs[0]), int(timeSegs[1]), int(float(timeSegs[2])))
    return timestamp_start

if __name__ == '__main__':
    main()


