"export" "PGENGINE=/usr/local/pgsql/bin"
"export" "MAILTO=root"
"export" "PATH=/sbin:/usr/sbin:/bin:/usr/bin:/usr/local/bin:/usr/bp/bin:/usr/bp/lib/shlib:/root/bin:/usr/local/pgsql/bin"
"export" "SHELL=/bin/bash"
"export" "PYTHONPATH=/usr/bp/lib/monitoring:/usr/bp/bin"
"export" "HOME=/"

#__________________________________________________________________________________________________
#author    : venkat@unitrends.com
#purpose : Create configuration with db notification
#Bug#        Date        Who Description
#------- ------- --- -----------------------------------------------------------------
#18040     14Mar14 VSK 

# Robert Underwood changed indenation and changed descriptions to docstrings
"exec" "python" "-u" "-Wignore" "$0" "$@"

import sys, os    
if not '/usr/bp/lib/monitoring' in sys.path:
	sys.path.append('/usr/bp/lib/monitoring')
try:
    import dbmon_Util as pgdb 
except:
    sys.exit(1)

def proc_cmdline():
    """
    command line parser to get basics options for standalone mode

    Keyword Parameters: 
    none pulls from sys.argv[]
    
    Keyword Returns:
    optparse.Values Options - object that contains the parsed arguments
    string[] Args - list that contains arguments that were not parsed
    optparse.OptionParser handle - instance of the optparse object used to parse
    """
    from optparse import OptionParser
    curdir = os.getcwd()
    usage = "%prog [options] arg"
    parser = OptionParser(usage)
    parser.add_option("-f", "--configPath", type="string", dest="configPath", default='/usr/bp/schema', 
            help="path+filename to monitoring config [default=%default]")
    parser.add_option("-c", "--category", type="string", dest="catIDRange", default='0,100', 
            help="category_id range [default=%default]")
    parser.add_option("-i", "--notifID", type="string", dest="notifIDRange", default='0,1000', 
            help="notification_id range [default=%default]")
    (options, args) = parser.parse_args()
    usr_args = []
    joboptions = []
    if len(args) > 0:
        for arg in args:
            if arg[0] != "-":
                joboptions.append(arg)
            else:
                usr_args.append(arg)
    return (options,args,parser)

def runQueryValidateData(query, count_query, csvFile):
    """
    runs a postgres query on the database

    Keyword Arguments
    string query - contains a valid postgres sql query
    string count_query  - contains a matching query that returns number of rows matched
    FILE cvsFile - a temporary csv file that holds the database output. NOTE THE QUERY MUST
        REDIRECT OUTPUT INTO THIS FILE!! Failure do so will cause errors

    returns
    int status - error code contains 1 if no errors 0 if there was an exception
    string[][] data - list of lists containing trap properties
    """
    status = 1 ## all ok
    num_records = -1 
    data = None

    ## find number of records returned by query
    out, err, p, res = pgdb.runCommand(count_query)
    if err == '' and p.returncode == 0:
        try:
            num_records= int(out.strip())
        except:
            status = 0

    if status == 0:
        return (status, data)

    csvok = False

    ## convert records returned by query (if any) into 
    ## equivalent config-type dict object
    out, err, p, res = pgdb.runCommand(query)
    if err == '' and p.returncode == 0 and os.path.isfile(csvFile):
        data = pgdb.readSamples(csvFile)
        if data:
            #print len(data), num_records
            csvok = (len(data) == num_records)
            if not csvok:
                status = 0

    ## remove csvFile
    cmd = 'rm -f %s > /dev/null 2>&1' % csvFile
    os.system(cmd)

    ## return status and data
    return (status, data)

def writeConfigData(configFile, mode, data, uniqID, orderBy):
    """
    Writes the configuration data to a file

    Keyword Arguements:
    Path configFile -  path to write the configuration file to
    string mode -  prefered mode to open the file if the file is 
        bigger than max_size_bytes will use "w" instead.
    string[][] data - list of lists containing the data to be written
    string uniqID - string that prefixes each unique id
    string orderBy = method of ordering, should match the header of the column

    Returns:
    int status - 1 if there was an error and 0 if there was not

    Also writes the configuation file to configFile
    """
    status = 0
    wrote_ok = 1
    max_size_bytes = 1048576
    try:
        cfgfile = pgdb.setupLogger(configFile, max_size_bytes, mode)
        if not cfgfile:
            return status

        if mode == 'w':
            tcn = pgdb.TimeConvert()
            ver = pgdb.Platform()
            cfgfile.write('\n;'+'-'*68);
            cfgfile.write('\n; Configuration extracted from database')
            cfgfile.write('\n; DO NOT MODIFY. Generated automatically by '+os.path.basename(__file__))
            cfgfile.write('\n; VERSION '+str(ver.getSWRelease())+' DATE '+str(tcn.TimeNow()))
            cfgfile.write('\n;'+'-'*68);
        for i in range(len(data)):
            try:
                if orderBy == None:
                    cfgfile.write('\n['+uniqID+str(i)+']')
                else:
                    cfgfile.write('\n['+uniqID+data[i][orderBy]+']')
                for j in data[i].keys():
                    cfgfile.write('\n'+j+': '+data[i][j]+'')
            except:
                cfgfile.close()
                os.remove(configFile)
                wrote_ok = 0
                break
        if cfgfile:
            endStr = '\n; '+uniqID+'_END\n;'
            cfgfile.write(endStr+len(endStr[4:])*'-')
            cfgfile.close()
        if wrote_ok==1:
            status = 1
    except:
        pass
    return status

def getConfigData(configFile, qParams, limit=1000):
    """
    get tables from database, parse canonical/delimeted csv and store results in a config object
    
    Keyword Arguements:
    Path configFile - path to the location the temporary file will be stored
    string[] qParams - options for the query that contains:
        string catIDRange - a string of the form 'a,b' where a and b are integers
            where a is the first category to check and b is the last category to check
        string notifIDRange - a string of the form 'a,b' where a and b are integers 
            where a is the first notification_id to check and b is the last notification_id to check
    int limit - maximum number of entries that will be reported from the postgres query

    Returns:
    returns -1 if there was an error and 1 if there was not an error
    """
    status = 0
    if len(qParams) != 2: 
        return -1

    catidrange = None
    notifidrange = None
    try:
        catidrange = sorted(map(int, filter(None, ("".join(qParams[0].split())).split(','))[0:2]))
        notifidrange = sorted(map(int, filter(None, ("".join(qParams[1].split())).split(','))[0:2]))
    except:
        pass
    if not (catidrange or notifidrange):
        return -1

    dbu = pgdb.DBUtil()
    if not dbu.pingDB():
        return -1

    tab = pgdb.SNMPTables;
    ## get notifications: Form the query to extract records matching requested ID's / categories
    csvFile=os.path.join('/tmp/.', os.path.splitext(os.path.basename(configFile))[0]
            + tab.name[tab.notifications] + '.csv')
    query    = 'psql bpdb postgres -XA -P footer=off -c '
    query += '"SELECT * from bp.notifications WHERE '
    if len(catidrange) == 2:
        query += 'category_id BETWEEN %d AND %d' % (catidrange[0], catidrange[1])
    elif len(catidrange) == 1:
        query += 'category_id = %d' % catidrange[0]
    else:
        query += '1=1 AND '

    if len(catidrange) > 0:
        query += ' AND '

    if len(notifidrange) == 2:
        query += 'notification_id BETWEEN %d AND %d LIMIT %d;"' % (notifidrange[0], notifidrange[1],limit)
    elif len(notifidrange) == 1:
        query += 'notification_id = %d LIMIT %d;"' % (notifidrange[0],limit)
    else:
        query += '1=1 LIMIT %d;"' % limit
    
    count_query = query.replace("*", "count(*)",1)
    count_query = count_query.replace("-XA", "-XAt",1) 
    query += '> %s 2>/dev/null' % csvFile
 
    ## run query, get output records, parse csv and obtain config data
    (status, data) = runQueryValidateData(query, count_query, csvFile)
    if status == 0 or not data:
        return -1
 
    uniqID = tab.uniqID[tab.notifications] 
    mode = 'w'
    status = writeConfigData(configFile, mode, data, uniqID, 'notification_id')
    if status != 1:
        return -1

    ## get all trap_types
    csvFile=os.path.join('/tmp/.', os.path.splitext(os.path.basename(configFile))[0]
            + tab.name[tab.trap_types] + '.csv')
    query    = 'psql bpdb postgres -XA -P footer=off -c "select * from bp.trap_types;"'
    count_query = query.replace("*", "count(*)",1)
    count_query = count_query.replace("-XA", "-XAt",1)
    query += '> %s 2>/dev/null' % csvFile

    (status, data) = runQueryValidateData(query, count_query, csvFile)
    #print data
    if status == 0 or not data:
        return -1

    uniqID = tab.uniqID[tab.trap_types] 
    mode = 'a'
    status = writeConfigData(configFile, mode, data, uniqID, 'trap_type_id')
    if status != 1:
        return -1
    
    ## get all categories
    csvFile=os.path.join('/tmp/.', os.path.splitext(os.path.basename(configFile))[0]
            + tab.name[tab.categories] + '.csv')
    query    = 'psql bpdb postgres -XA -P footer=off -c "select * from bp.categories;"'
    count_query = query.replace("*", "count(*)",1)
    count_query = count_query.replace("-XA", "-XAt",1)
    query += '> %s 2>/dev/null' % csvFile

    (status, data) = runQueryValidateData(query, count_query, csvFile)
    if status == 0 or not data:
        return -1

    uniqID = tab.uniqID[tab.categories]
    mode = 'a'
    status = writeConfigData(configFile, mode, data, uniqID, 'category_id')
    if status != 1:
        return -1
    
    ## get host-type
    csvFile=os.path.join('/tmp/.', os.path.splitext(os.path.basename(configFile))[0]
            + tab.name[tab.nvp] + '.csv')
    query    = 'psql bpdb postgres -XA -P footer=off -c "select * from bp.nvp where item_name=\'Identity\';"'
    count_query = query.replace("*", "count(*)",1)
    count_query = count_query.replace("-XA", "-XAt",1)
    query += '> %s 2>/dev/null' % csvFile

    (status, data) = runQueryValidateData(query, count_query, csvFile)
    if status == 0 or not data:
        return -1
    uniqID = tab.uniqID[tab.nvp]
    mode = 'a'
    status = writeConfigData(configFile, mode, data, uniqID, 'item_name')
    if status != 1:
        return -1
 
    ## get server-config
    csvFile=os.path.join('/tmp/.', os.path.splitext(os.path.basename(configFile))[0]
            + tab.name[tab.snmp_config] + '.csv')
    query    = 'psql bpdb postgres -XA -P footer=off -c "select * from bp.snmp_config where enabled=true;"'
    count_query = query.replace("*", "count(*)",1)
    count_query = count_query.replace("-XA", "-XAt",1)
    query += '> %s 2>/dev/null' % csvFile

    (status, data) = runQueryValidateData(query, count_query, csvFile)

    orderBy = None 
    if status == 0 or not data:
        return -1

    uniqID = tab.uniqID[tab.snmp_config]
    mode = 'a'
    status = writeConfigData(configFile, mode, data, uniqID, orderBy)
    if status != 1:
        return -1

    return status


def tableDump(options, args, parser):
    """
    dump contents of SNMP specific tables into a persistent config file

    Keyword Arguemnts:
    optparse.Values Options - object that contains the parsed arguments
    Args - list that contains arguments that were not parsed
    optparse.OptionParser Parser - the object parser object
    
    Returns:
    returns 0 if there was not an error and -1 if there was an error
    """
    import inspect
    idself = inspect.stack()[0][3]

    ## set defaults
    configPathFile=os.path.join('/usr/bp/schema','.dbmon_'+idself)
    catIDRange = '0,100'
    notifIDRange = '0,1000'

    ## load cmdline options
    if options.configPath:
        configPathFile = os.path.join(options.configPath, '.dbmon_'+idself)

    if options.catIDRange:
        catIDRange = options.catIDRange

    if options.notifIDRange:
        notifIDRange = options.notifIDRange
 
    qParams = [ catIDRange, notifIDRange ]

    ## update configuration
    upd_status = getConfigData(configPathFile, qParams)

    ## if update succeeds, change file permissions end exit
    if upd_status == 1 and os.path.isfile(configPathFile):
        os.chmod(configPathFile,0444)
        return 0
    return upd_status

#__________________________________________________________________________________________________
## py conditional script hook
if __name__ == "__main__":
    options, args,parser = proc_cmdline()
    sys.exit(tableDump(options,args,parser))
