#!/usr/bin/env python

#import the standard library modules that are needed
import subprocess , string , os , os.path , sys , ConfigParser , datetime
from optparse import OptionParser
import datetime

# Ensure the framework is in the path
if not "/usr/bp/lib/monitoring" in sys.path:
	sys.path.append("/usr/bp/lib/monitoring")

# Set up logging
import logging
logger  = logging.getLogger('dynamic_manager')
logger.setLevel(logging.DEBUG)

# Set up the file handler
logDirectory = '/var/log/monitoring_onDemandTrap'  
handler = logging.FileHandler(os.path.join(logDirectory,'dynamic_manager.log'))
handler.setLevel(logging.INFO)
formatter = logging.Formatter('%(asctime)s - %(name)s - %(levelname)s - %(message)s')
handler.setFormatter(formatter)
logger.addHandler(handler)

# Set up the stream handler for debugging purposes; it should be commented out
# for production

# streamhandler = logging.StreamHandler()
# streamhandler.setLevel(logging.ERROR)
# streamFormatter = logging.Formatter('%(message)s')
# streamhandler.setFormatter(streamFormatter)
# logger.addHandler(streamhandler)

# Import the other framwork scripts that are needed
import verify
import cron
from dbmon_Util import runCommand

#Begrudgingly re-adding this import to fix Woochun's command line options
#pending further testing where they can be isolated from the import *

# introduces several globals that polute the global namespace including dirname
from util import * 

#Module constants
public_key_path = '/usr/bp/bin/monitoring/onDemand/public.pem'
cron_path = '/usr/bp/bin/monitoring/onDemand/crontab'

class OnDemandTask:
	"""
	Trap specification for the OnDemand Trap framework
	"""
	#previously called Section


	#why is this a global variable?
	global defaultSection

	def __init__(self, config, name):
		"""
		Uses a ConfigParser to read trap specification from the ini file

		Keyword Parameters
		ConfigParser config - parser that does the actual reading of the values
		string name - name of the task which should match its section in the ini file

		Returns:
		Void
		"""

		self.task		= name
		self.config	  = config
		self.Version	 = self.getValue("Version")
		self.FtpPath	 = self.getValue('FtpPath'  )
		self.LocalPath   = self.getValue('LocalPath')
		self.Script		 = self.getValue('Script'   )
		self.Frequency   = self.getValue('Frequency')
		self.StartDate   = self.getValue('StartDate')
		self.EndDate	 = self.getValue('EndDate'  )

	def getValue(self, option):
		"""
		getter function that interfaces with the ConfigParser

		Keyword Parameters:
		string option - option to look for in configuration file checks section specified
			in self.task first then checks the default section.

		Returns:
		String containing the value 
		"""
		try:
			value = self.config.get(self.task, option)
		except:
			value = ""
		return value

	def __repr__(self):
		"""
		Prints a summary of the task

		Keyword Parameters:
		None

		Returns:
		stirng in crontab format
		"""
		return "%10s  %s   %s %s %s   %s   [%s,%s]" \
			% (self.task, self.Version, self.FtpPath,
			self.LocalPath, self.Script, self.Frequency, self.StartDate, self.EndDate)

	def update(self, default):
		"""
		function is empty

		Keyword Parameters:
		?? default - ??

		Returns:
		no return type
		"""
		#to be implemented;  What is this supposed to do??
		return

def parse_args():
	"""
	Parses options and calls the appropriate

	Keyword Parameters:
	None; Reads arguments from stdin

	Returns:
	OptionParser.Values opts - contains the options that were passed
	string[] args - remaining arguments after parsing options

	returns (None,None) if there was a problem
	"""
	abs_path = os.path.abspath(sys.argv[0]) 
	basename = os.path.basename(sys.argv[0])
	dirname = abs_path.replace(basename, "") 

	parser = OptionParser(usage = 'usage: %prog [options]')

	parser.add_option("-d", "--debug",   dest="debug", default=False, action='store_true', 
		help="debug option to show detail commands to test")
	parser.add_option("-a", "--add", dest="add", default=0, action='store', nargs=2, 
		help="add SNMP destination with ipadd and hostname (for testing purpose) e.g. %s --add 10.10.10.1 MyHostName" %basename)
	parser.add_option("--disable", dest="disable", default=0, action='store', 
		help="disable SNMP destination e.g. %s --disable notifications.unitrends.com" %basename)
	parser.add_option("--snmp_config", dest="snmp_config", default=False, action='store_true', 
		help="show snmp_config table entries")
	parser.add_option("--install", dest="install", default=False, action='store_true', 
		help="install it in dpu for development")

	(opts, args) = parser.parse_args()
	return (opts, args)

def get_Cut_Levels(ftp_address):
	"""
	calculates the appropriate number of cut levels for the download_and_verify function

	Keyword Parameters:
	string ftp_address - path for the ftp_address

	Returns:
	int number of levels
	"""
	levels = len(ftp_address.strip().split('/')) -3
	if levels < 0:
		raise TypeError
	else:
		return levels

def is_safe_dir(dir, safe_dir):
	"""
	returns true if the dir is in the safe dir
	
	Keyword Parameters:
	sting dir - path to the directory to be verified
	string safe_dir - path to to the "safe" directory

	return: True if yes, False if no
	"""
	return (os.path.commonprefix([os.path.abspath(safe_dir), os.path.abspath(dir)]) == os.path.abspath(safe_dir))

def download_and_verify(LocalPath, FtpPath, StartDate = -1, EndDate = -1, isScriptDir=False):
	"""
	Download the scripts from the ftp location also does the clean up for scripts when
	they are old.  LocalPath must be in the safe directory for the scripts to be 
	downloaded.  

	Keyword arguments:
	string LocalPath - path to save the file(s) to
	string FtpPath - path to the file(s) on the ftp server
	StartDate - if today is before the start date delete the file inclusive
	EndDate - if today is after the end date delete the file inclusive
	isScript:Dir - additionally deletes the files in /usr/bp/bin/monitoring/onDemand/data/TASKNAME
		if the entire directory is empty if true

	Return:
	bool - returns True if there were no errors and False if there was an error
	"""
	# if the longest commonprefix of the path is not in the scripts directory
	# abort NOW and don not download the information for the script

	if(is_safe_dir(LocalPath, '/usr/bp/bin/monitoring/onDemand')):
		try:
			cut_levels = get_Cut_Levels(FtpPath)
		except TypeError:
			return False
		cmdOut , cmdErr, cmdPipe, cmdRes =  runCommand(("/usr/bin/wget -N %s -r -nH --cut-dirs=%s -P %s" %(FtpPath, cut_levels, LocalPath)))
		logger.debug("%s \n %s"% (cmdOut, cmdErr))
		if cmdPipe.returncode != 0:
				return False
		return verify_dir(LocalPath,StartDate,EndDate, True,isScriptDir)
	else: return False

def valid_file(infile, signature,StartDate,EndDate):
	"""
	Checks if a file is in date, has a valid signature, etc

	Keyword Arguments:
	String infile: path to the file to verify
	String signature: path to the signature file
	Int StartDate: the date in the format YYYYMMDD or -1 to ignore
	Int EndDate: the date in the format YYYYMMDD or -1 to ignore

	Return:
	bool is_valid - returns True if a file verifies or false if it does not
	"""
	#This is fairly small and unimpressive right now, but it will have more in here later
	StartDate = int(StartDate)
	EndDate = int(EndDate)

	#Verify that the signature is valid
	logger.debug("Verifing file: " + infile)
	is_valid = verify.verify(infile,public_key_path,signature)
	logger.debug("--> signature verifies : %s"% is_valid)

	#Verify that the file is in date
	today = int(datetime.date.strftime(datetime.date.today(), "%Y%m%d"))
	is_valid = is_valid and (((StartDate == -1) or (StartDate <= today)) and ((EndDate == -1) or (today <= EndDate)))
	logger.debug("--> dates verifies: %d, %d, %d %s"%(StartDate,today , EndDate	, is_valid))

	#To be extended as follows:
	#is_valid = is_valid and other_criterion()
	return is_valid

def verify_dir(local,StartDate,EndDate, CONFIRM=False, isScriptDir=False):
	"""
	Verifies all of the files in the directory specified with the provided public key

	!!!WARNING!!! -------------------------------------------------------------------
	THIS FUNCTION WILL DELETE ALL FILES IN THE DIRECTORY SPECIFIED AS LOCAL IF THEY
	DO NOT PASS THE VERIFICATION ROUTINES!!
	---------------------------------------------------------------------------------

	Keyword Arguments:
	string local - path to the scripts directory
	int StateDate - earliest date that the code is valid
	int EndDate - latest date that the code is valid
	bool CONFIRM - set to true to actually execute; assertion fail if false
	bool isScriptDir - if true also clean up data files and the folder retrieved

	Returns:
	True if all files verified successfully

	"""
	assert(CONFIRM == True)
	if not os.path.exists(local): 
		return False

	flag = True
	sigs = []
	scripts = []
	directories = []
	for path, dirs, files in os.walk(local):
		for dir in dirs:
			directories.append(os.path.join(path,dir))
		for file_path in files:
			if os.path.islink(os.path.join(path,file_path)):
				continue
			if (file_path[-4:] == ".sig"):
				# this file is likely a signature
				sigs.append(os.path.join(path, file_path))
			else:
				# the file is likely a script or resource
				scripts.append(os.path.join(path, file_path))

	for sig in sigs:
		if sig[:-4] not in scripts:
			# this may be a malicious signature file that actually contains a script
			# delete the signature file

			os.remove(sig)
			pass
		else:
			# remove the script from the scripts array this is for later clean up
			scripts.remove(sig[:-4])
			if not valid_file(sig[:-4], sig,StartDate,EndDate):
				# the script did not verify and may be compromised
				# delete the signature and the script
				flag = False
				os.remove(sig)
				os.remove(sig[:-4])
			else:
				os.chmod(sig[:-4],0744)

	for script in scripts:
		# if there is a script that is still in this array it may be malicious
		# remove it from the system
		os.remove(script)
	for dir in directories:
		#Clean up sub directories if they are empty
		try:
			os.rmdir(dir)
		except OSError:
			pass
	if isScriptDir :
		#Clean up top level directores if the files are scripts
		try:
			os.rmdir(local)
		except OSError:
			pass
		if not os.path.exists(local):
			#If all of the script files are gone delete the datafiles
			data_file_path = os.path.join('/usr/bp/bin/monitoring/onDemand/data', os.path.basename(local))
			# Using topdown false to clean up data files and directories in one pass
			for path, dirs, files in os.walk(data_file_path, topdown=False):
				for data_file in files:
					os.remove(os.path.join(path,data_file))
				for directory in dirs:
					os.rmdir(os.path.join(path,data_file))
			# remove the root data file for a script if it is empty
			try:
				os.rmdir(data_file_path)
			except OSError:
				pass

	return flag

def update_tasks():  # including wget 
	"""
	gets all of the tasks and puts them into a python list

	Keyword Arguements:
	None

	Returns:
	onDemandTasks[] - all of the on traps that are currently requested
	"""
	logger.info("Entered Update")

	#Constants
	Public_key_ftp_path = 'ftp://ftp.unitrends.com/utilities/onDemand/Public/'
	Public_key_local_path = '/usr/bp/bin/monitoring/onDemand/public'
	configFile = '/usr/bp/bin/monitoring/onDemand/config/onDemandConfig.ini' 
	test_ini_ftp_path = 'ftp://ftp.unitrends.com/utilities/onDemand/config'

	# Check for an update to the Public Key
	logger.info("Checking for new public keys")
	download_and_verify(Public_key_local_path, Public_key_ftp_path)
	if (os.path.exists(os.path.abspath('/usr/bp/bin/monitoring/onDemand/public'))):
		# creates a list of the Public keys that survive verification
		Keys = [files for files in os.listdir(Public_key_local_path) \
			if os.path.isfile(os.path.join(Public_key_local_path,files))]
		logger.debug("Keys %s" %Keys)
		Keys.sort()
		if len(Keys) == 2:
			logger.info("Public Keys were updated")
			os.rename(os.path.join(Public_key_local_path,Keys[0]),public_key_path)
	if not os.path.exists(public_key_path): return []
	
	logger.info("Checking for new config file")
	

	# Download the configuration file
	logger.info('Checking for new configuration file')
	download_and_verify(os.path.dirname(configFile), test_ini_ftp_path)
	
	# get_tasks returns an empty list if the configFile does not exist
	tasks = get_tasks(configFile);
	if tasks == []:
		return tasks

	logger.info('Updating cronfile')
	crontab = cron.cronManager(cron_path)
	for task in tasks:
		download_and_verify(task.LocalPath, task.FtpPath, task.StartDate, task.EndDate,True)
		logger.info("TASK: %s" % task)
		if is_safe_dir(task.Script, task.LocalPath) and os.path.exists(task.Script):
			try:
				crontab.insert(task.task,task.Frequency,task.Script)
			except cron.DuplicateJobError:
				crontab.update(task.task,task.Frequency,task.Script)
			result, error, pipe, rusage  = runCommand(task.Script)
			logger.debug("%s\n%s" %(result, error))
		else:
			logger.info("Script was not in the SAFE DIR or does not exist")
		if int(task.EndDate) <= int(datetime.date.strftime(datetime.date.today(), '%Y%m%d')) and int(task.EndDate) != -1:
			try:
				crontab.remove(task.task)
			except cron.NonExistantJobError:
				pass
	crontab.write()

	return tasks

def get_tasks(configFile):
	"""
	gets the tasks given the configuraton file BUT does not get updates
	
	Keyword Arguments
	string configFile - file path to the onDemand configuration file

	returns onDemandTasks[] containing each task in the config file
	"""
	config = ConfigParser.ConfigParser()
	config.read(configFile)
	retrieved_tasks = []

	global defaultSection 

	defaultSection = OnDemandTask(config, 'DEFAULT')
	logger.info("loading all tasks from the config file")
	for task in config.sections():
		theOnDemandTask = OnDemandTask(config, task)
		retrieved_tasks.append(theOnDemandTask)
		
	return retrieved_tasks 

def init():
	"""
	Preform variable setup before Dynamic Traps can be used
	Keyword Parameters:
	None

	Returns:
	bool - returns false if another option was passed or there was a problem initializing
	"""
	try:
		if not os.path.exists(logDirectory):
			os.system("mkdir %s" % logDirectory)
	except:
		logger.error("Could not create the log directory file")
		return False
	if not os.path.exists('/etc/cron.d/onDemandCron'):
		try:

			# Ensure that the cron file exists and can be edited before symlinking it
			open(cron_path,'a').close()
			os.symlink(cron_path,'/etc/cron.d/onDemandCron')
		except:

			# Cronfile and symlink are needed to run the cron manager; Abort
			logger.error("Could not create the symlink to the crontab file")
			return False
	return True

def main():
	"""
	Main function for dynamic trap management program

	Returns: 0 on successful execution and 1 if there was a problem
	"""
	if not init():
		return 1

	(opts, args) = parse_args()

	if opts.snmp_config:
		dests = get_db_entries("snmp_config")
		printArray(dests)
		return 0

	elif opts.add:
		ipaddr = opts.add[0]
		hostname = opts.add[1]
		lines, _, _,_ = runCommand("/usr/bp/bin/cmc_network host info %s" % ipaddr)
		doesExist = (len(lines)>=2)
		
		if doesExist:
			runCommand("/usr/bp/bin/cmc_network host save %s %s %s" % (ipaddr, ipaddr, hostname))
		else:	
			runCommand("/usr/bp/bin/cmc_network host save - %s %s" % (ipaddr, hostname))

		run_query(("INSERT INTO bp.snmp_config (destination, community, enabled) "
			+ "VALUES ('%s','public','t')") % hostname)	

		return 0

	elif opts.disable:
		run_query("UPDATE bp.snmp_config SET enabled='f' WHERE destination = '%s'" % opts.disable)
		return 0

	elif opts.install:
		if CONFIG.majorVersion<7 :
			logger.error( "installation is not supported before version 7.X")
			return 1

		if dirname == "/root/benchmark/onDemand/": # only if supporting case
			os.system("cp %s /usr/bp/bin/" % basename)
			os.system("cp /root/benchmark/diskmon/lib/*py /usr/bp/lib/monitoring/")
			logger.info( "%s is installed in /usr/bp/bin/ and lib in /usr/bp/lib/monitoring/" %basename)
			return 0
		else:
			logger.error( "Installation is available only from %s " % dirname)
			return 1
	else:
		#default functionality
		tasks = update_tasks()
		if tasks == [] : 
			logger.info("Tasks was None!")
		
		return 0

if __name__ == '__main__':
	main()


