import time
# to remove new lines: regexp_replace(object,E'[\\n\\r]+', ' ', 'g' )
from config      import *   

def GetAlerts(command):
   alerts =[]
   if not CONFIG.dbON:
      return alerts
   
   import commands      
   lines = commands.getoutput(command).splitlines()
   for line in lines:
       alerts.append(alert(line))
   return alerts

def GetTraps(command="/usr/bin/psql -U postgres bpdb -tq -c \"select id,oid,time_sent,destination,community,traptype,regexp_replace(object,E'[\\n\\r]+', ' ', 'g' ),severity,status,description from bp.snmp_history order by id desc\" | grep -v ^$"):
   traps =[]
   if not CONFIG.dbON and not "dbout" in command:
      return traps
   
   import commands
   lines = commands.getoutput(command).splitlines()
   for line in lines:
       traps.append(snmp_history(line))
   return traps

def str2bool(v):
    return v.lower() in ("yes", "true", "t", "1")

class snmp_config:
    def __init__(self, line):
        values = line.split("|")
        self.id = int(values[0])
        self.destination = values[1].strip()
        self.community   = values[2].strip()
        self.enabled = str2bool(values[3].strip())

    def __repr__(self):
        return "id: %2d   destination: %30s   community: %s   enabled: %s" % (self.id, self.destination, self.community, self.enabled)

        # psql -U postgres bpdb -c "select * from bp.categories"
        # category_id |  category
        # -------------+-------------
        #            0 | Internal
        #            1 | Platform
        #            2 | Storage
        #            3 | Core
        #            4 | Backup
        #            5 | Restore
        #            6 | Management
        #            7 | Archiving
        #            8 | Vaulting
        #           15 | Disk Health
        #           16 | IPMI Event

class category:
    def __init__(self, category_id=-1, category="", line=""):

        self.category_id = category_id
        self.category    = category
        if len(line)>0:
            values = line.split("|")
            self.category_id = int(values[0])
            self.category = values[1].strip()
            
    def __repr__(self):
        return "category_id: %2d  is %10s" % (self.category_id, self.category)

        # psql -U postgres bpdb -c "select * from bp.trap_types order by trap_type_id"
        #trap_type_id |           trap_type
        #-------------+-------------------------------
        #1 | Clients state has changed
        #2 | 3ware state has changed
        #3 | PCI state has changed
        #5 | Process state has changed
        #6 | Disk state has changed
        #7 | CEP state has changed
        #8 | System state has changed
        #9 | RAID state has changed
        #10 | Backup state has changed
        #11 | Archiving state has changed
        #12 | Version Information
        #13 | System Open Alerts Status
        #14 | Network Status has changed
        #15 | Disk health state has changed
        #16 | IPMI Event has occurred
        #99 | Test Trap

class trap_type:
    def __init__(self, trap_type_id=-1, trap_type="", line=""):
        self.trap_type_id = trap_type_id
        self.trap_type    = trap_type

        if len(line)>0:
            values = line.split("|")
            self.trap_type_id = int(values[0])
            self.trap_type = values[1].strip()
    def __repr__(self):
        return "trap_type_id: %2d  is %10s" % (self.trap_type_id, self.trap_type)


class alert:
    def __init__(self, line):
        values = line.split("|")
        self.alert_id   = int(values[0])
        self.alert_text = values[1].strip()
        self.source_id  = int(values[2])
        self.target     = values[3].strip()
        self.severity   = int(values[4])
        self.created    = values[5] # too long for int??
        self.updated    = values[6] # too long for int??
        self.closed     = values[7].strip()
        self.notification_id     = int(values[8])

    def __repr__(self):
        return "alert_id: %2d  is %10s" % (self.alert_id, self.alert_text)

class snmp_history:
    def __init__(self, line):
       print line
       values = line.split("|")
       self.id                   = int(values[0])
       self.oid                  = values[1]
       words = values[1].split(".")
       self.oidIndex             = int(words[len(words)-1])

       time_sent = values[2].strip()
       # 2013-01-07 12:32:20.105695 =>          2013-01-07 12:32:20
       time_sent= time_sent[0:19]
       self.time_sent_epoch = time.mktime(time.strptime(time_sent, "%Y-%m-%d %H:%M:%S")) #converted to epoch
       
       from datetime import datetime
       timeSent = datetime.fromtimestamp(self.time_sent_epoch)
       self.time_sent =timeSent
       self.time_sent_hour = timeSent.hour
       self.time_sent_day  = timeSent.strftime("%a") #Sun, Mon....
       
       self.destination          = values[3]
       self.community            = values[4]
       self.traptype             = values[5]
       self.object               = values[6].strip()
       self.severity             = int(values[7])
       self.status               = int(values[8])  #0 means closed and 1 means open
       self.description          = values[9].strip()
       
    def __repr__(self):
        return "%d %d %s %d %s %s %s %s %s" \
            % (self.id, self.oidIndex, self.time_sent_day, self.time_sent_hour, \
                  self.destination, self.community, self.traptype, self.object, self.description)

class backup:
    """psql command line return; so you have to digest it here"""
    def __init__(self, line):
        values = line.split("|")
        self.backup_no      = int(values[0])
        self.node_no        = int(values[1])
        self.start_time     = int(values[2])
        self.elapsed_time   = int(values[3])
        self.properties_raw = int(values[4])
        self.properties     = BackupProperties(int(values[4]))
        self.total_netpaths = int(values[5])
        self.total_megs     = int(values[6])
        self.status         = int(values[7])
        self.type           = BackupType(int(values[8]))
        self.throughput = 0
        self.start_htime = time.strftime('%Y-%m-%d %H:%M:%S', time.localtime(self.start_time))
        self.end_htime = time.strftime('%Y-%m-%d %H:%M:%S', time.localtime(self.start_time+self.elapsed_time))

        if self.elapsed_time > 0: 
            self.throughput = self.total_megs/float(self.elapsed_time)

        #self.node_name = "NA"    
        #for node in nodes:
        #    if node.node_no == self.node_no:
        #        self.node_name = node.node_name
        #        break
        return

    def __repr__(self):
        head = "bckno %3d" % self.backup_no  
        for i in range(13-len(head)):
            head = head + " "
        return head + "(%d)  ++++ " % self.node_no +  "   elapsed_time: %6d " % self.elapsed_time +\
            "   total_megs: %7d " % self.total_megs + "   total_netpaths: %7d " % self.total_netpaths +\
            " => %5.2f(MB/s)" % self.throughput  + "    %s " % self.type +\
            "   started: %s" % self.start_htime + "  properties: %s " % self.properties  
    
class backup_file:
    def __init__(self, line):
        values = line.split("|")
        self.backup_no = int(values[0])
        self.end_backup_no = int(values[1])
        self.dir_no  = int(values[2])
        self.fname   = values[3].strip()
        self.type    = int(values[4])
        self.size    = float(values[5])/1024. # kB
        
    def __repr__(self):
        return "%i size(kB): %f" %(self.backup_no, self.size)

class BackupType:
    def __init__(self, type):
        self.raw = type

        self.type = "         "    
        if   type ==    1 : self.type = "Master   "
        elif type ==    2 : self.type = "Increment"
        elif type ==    3 : self.type = "Incr4ever"
        elif type ==    4 : self.type = "Selective"
        elif type ==    5 : self.type = "EXCHGFULL"
        elif type ==    6 : self.type = "EXCHGDIFF"
        elif type ==    7 : self.type = "EXCHGINCR"
        elif type ==    9 : self.type = "MSSQLFULL"
        elif type ==   10 : self.type = "MSSQLDIFF"
        elif type ==   11 : self.type = "MSSQLTRNS"
        elif type ==   12 : self.type = "VMESXFULL"
        elif type ==   13 : self.type = "VMESXDIFF"
        elif type ==   14 : self.type = "VMESXINCR"
        elif type == 1031 : self.type = "HYPRVFULL"  #0x407
        elif type == 1032 : self.type = "HYPRVDIFF"       
        elif type == 1033 : self.type = "HYPRVINCR"         
        elif type == 1034 : self.type = "ORACLFULL"  #0x40A
        elif type == 1035 : self.type = "ORACLINCR"         
        elif type == 1036 : self.type = "SHAREFULL"         
        elif type == 1037 : self.type = "SHAREINCR"  #0x40D
        elif type == 1038 : self.type = "APPLEMETA"
        elif type == 1039 : self.type = "RAE__FULL"  #0x40F
        elif type == 4096 : self.type = "RAE__DIFF"  #0x1000
        elif type == 4097 : self.type = "RAE__INCR"
        elif type == 4098 : self.type = "NDMP_FULL"
        elif type == 4099 : self.type = "NDMP_DIFF"
        elif type == 4100 : self.type = "NDMP_INCR"
        elif type ==   16 : self.type = "Restore  "

    def __repr__(self):
        return self.type

class BackupProperties:
    def __init__(self, properties):
        self.raw = properties

        self.properties = []
              
        if properties & 0x00000001:            self.properties.append("SYNTHETIC")
        if properties & 0x00000002:            self.properties.append("COMP_HARD")
        if properties & 0x00000004:            self.properties.append("COMP_SOFT")
        if properties & 0x00000008:            self.properties.append("WPROTECTD")
        if properties & 0x00000010:            self.properties.append("NO_DETAIL")
        if properties & 0x00000020:            self.properties.append("IMPORTED ")
        if properties & 0x00000040:            self.properties.append("P_COPY   ")
        if properties & 0x00000080:            self.properties.append("P_SQL    ")
        if properties & 0x00000100:            self.properties.append("ENCRYPTED")
        if properties & 0x00000200:            self.properties.append("HASHED   ")
        if properties & 0x00000400:            self.properties.append("DDUPCATED")
        if properties & 0x00000800:            self.properties.append("SYNTHESZD")
        if properties & 0x00001000:            self.properties.append("SYNTCAPBL")
        if properties & 0x00002000:            self.properties.append("DISK_META")
        if properties & 0x00004000:            self.properties.append("REPLICATD")
        if properties & 0x00008000:            self.properties.append("CLNTCONFG")
        if properties & 0x10000000:            self.properties.append("P_ODIRECT")
        if properties & 0x20000000:            self.properties.append("VMWRTMPLT")

        log = ""
        for prop in self.properties:
            log = log + "%s " % prop 

        self.words = log
        
    def __repr__(self):
        return self.words



class NodeType:
    def __init__(self, properties):
        self.raw = properties
        self.properties = "NA" 
        #print properties
        if properties == 0x0001:            self.properties="DOS"
        if properties == 0x0002:            self.properties="WIN16"
        if properties == 0x0003:            self.properties="NT"
        if properties == 0x0004:            self.properties="W95"
        if properties == 0x0005:            self.properties="OS2"
        if properties == 0x0006:            self.properties="NOV3"
        if properties == 0x0007:            self.properties="UNIX3"
        if properties == 0x0008:            self.properties="ODT"
        if properties == 0x0009:            self.properties="OS5"
        if properties == 0x000A:            self.properties="UNIX4"
        if properties == 0x000B:            self.properties="SOLARIS"
        if properties == 0x000C:            self.properties="SUNOS"
        if properties == 0x000D:            self.properties="NOV4"
        if properties == 0x000E:            self.properties="OSF1"
        if properties == 0x000F:            self.properties="NTS"
        if properties == 0x0010:            self.properties="AIX"
        if properties == 0x0011:            self.properties="SGI"
        if properties == 0x0012:            self.properties="DGUX"
        if properties == 0x0013:            self.properties="HP1"
        if properties == 0x0014:            self.properties="LINUX"
        if properties == 0x0015:            self.properties="BSDI"
        if properties == 0x0016:            self.properties="USL"
        if properties == 0x0017:            self.properties="UW"
        if properties == 0x0018:            self.properties="FBSD"
        if properties == 0x0019:            self.properties="MACOS"
        if properties == 0x001A:            self.properties="NOV5"
        if properties == 0x001B:            self.properties="NOV6"
        if properties == 0x001C:            self.properties="NOV65"
        if properties == 0x001D:            self.properties="2K"
        if properties == 0x001E:            self.properties="XP"
        if properties == 0x001F:            self.properties="2K3"
        if properties == 0x0020:            self.properties="VISTA"
        if properties == 0x0021:            self.properties="IBMI5"
        if properties == 0x0022:            self.properties="OES"
        if properties == 0x0023:            self.properties="WIN2K8"
        if properties == 0x0024:            self.properties="WIN2K8R2"
        if properties == 0x0025:            self.properties="WIN7"
        if properties == 0x0026:            self.properties="WIN8"
        if properties == 0x0027:            self.properties="WIN2K12"
        if properties == 0x0028:            self.properties="GENERIC"
        if properties == 0x0029:            self.properties="WIN81"
        if properties == 0x002A:            self.properties="WIN2K12R2"
    def __repr__(self):
        return self.properties

class application_instance:
    def __init__(self, line):
        values = line.split("|")
        self.instance_id          = int(values[0])
        self.node_no              = int(values[1])
        self.app_id               = int(values[2])
        self.current              = False
        if values[3].strip() == "t":
           self.current = True
        self.key1                 = values[4]
        self.key2                 = values[5]
        self.key3                 = values[6]
        self.key4                 = values[7]
        self.sync                 = False
        if values[8].strip() == "t":
           self.sync = True
        self.priority             = int(values[9])
        self.wvf                  = values[10]
        self.credential_id        = values[11]
        self.app_aware            = values[12]
        self.identity             = values[13]
        self.discovery_time       = int(values[14])

        #if self.node_no==129:
        #   print self.instance_id, self.app_id, values
    def __repr__(self):
       return "%d:%d (%d,%s)  %s %s %s %s"%(self.node_no, self.instance_id, self.app_id, self.current, self.key1, self.key2, self.key3, self.key4)
    
class operation_progress:
    def __init__(self, line):
        values = line.split("|")
        self.progress_id          = int(values[0])
        self.name_id              = int(values[1])
        self.start_time           = int(values[2])
        self.snapshot_time        = int(values[3])
        self.final_size           = int(values[4])
        self.current_size         = int(values[5])
        self.pid_to_kill          = values[6]
        self.on_behalf_of         = values[7]
        self.source               = values[8]
        self.destination          = values[9]
        self.backup_no            = int(values[10])
        self.backup_type          = values[11]

        self.start_htime = time.strftime('%Y-%m-%d %H:%M:%S', time.localtime(self.start_time))
        self.snapshot_htime = time.strftime('%Y-%m-%d %H:%M:%S', time.localtime(self.snapshot_time))

    def part(self):    
        return "         operation in progress at status of %10s " % operation_names[self.name_id-1].name_value +\
            "  final_size: %10d  << current_size: %10d " % (self.final_size, self.current_size) + " %s  %s " %(self.start_htime, self.snapshot_htime)

    def __repr__(self):
        return "backup_no: %5d (%10s) " % (self.backup_no, self.backup_type) +\
            "  name_id: %7s " % operation_names[self.name_id-1].name_value +\
            "  final_size: %10d  << current_size: %10d " % (self.final_size, self.current_size)

class backup_rep: # including node information
    """psql command line return; so you have to digest it here"""
    def __init__(self, line):
        values = line.split("|")
        self.node_no        = int(values[0])
        self.node_name      = values[1].strip()
        self.node_type      = int(values[2])
        self.os             = NodeType(int(values[3]))
        self.osid           = int(values[3])
        self.node_properties     = int(values[4])

        self.backup_no      = int(values[5])
        self.start_time     = int(values[6])
        self.elapsed_time   = int(values[7])
        self.properties_raw = int(values[8])
        self.backup_properties     = BackupProperties(int(values[8]))
        self.total_netpaths = int(values[9])
        self.total_megs     = int(values[10])

        addChar = ''
        if "SYNTHESZD" in "%s"%self.backup_properties:
            addChar ="STH"
        
        self.backup_type    = "%s%s" %(addChar, BackupType(int(values[11])))
        self.backup_type_raw    = int(values[11])
        self.instance_id    = -9
        
        if len(values[12].strip())>0:
           self.instance_id = int(values[12])
        self.throughput = 0
        self.start_htime = time.strftime('%Y-%m-%d %H:%M:%S', time.localtime(self.start_time))
        self.elapsed_htime = "%02dh%02dm%02ds" %( int(self.elapsed_time/3600), int( (self.elapsed_time%3600)/60 ), (self.elapsed_time%3600)%60)

        words = (self.start_htime.split()[1]).split(":")
        self.startHr = float(words[0]) + float(words[1])/60. + float(words[2])/60./60.
        self.endHr = self.startHr + float(self.elapsed_time)/60./60. + 0.1

        words = (self.start_htime.split()[0]).split("-")
        self.date = int(words[0])*10000 + int(words[1])*100 + int(words[2])

        if self.elapsed_time > 0: 
            self.throughput = self.total_megs/float(self.elapsed_time)

        self.sync = False
        self.app_id = -9
        return

    def fillAttribute(self, application_instances):
       for instance in application_instances:
          if instance.node_no != self.node_no: continue
          if instance.instance_id != self.instance_id: continue
          self.app_id = instance.app_id
          if instance.sync == "t": self.sync = True
          break
       return

    def __repr__(self):
        mess = "(%2d:%d) " % (self.node_no, self.instance_id)
        if self.sync:
            mess += "*"
        else:
            mess += " "
        mess += "%15s ++++ (%4d) %4d %8s %d   %04d  %8d  %5d   %s  ;; %s %d" % \
            (self.node_name, self.backup_no, self.node_type, self.os, \
                self.node_properties, self.backup_no, self.total_megs, self.backup_type_raw, self.backup_type, self.sync, self.app_id)    
        return mess

class replication_queue:
    def __init__(self, line):
        values = line.split("|")
        self.backup_no            = int(values[0])
        self.vault                = values[1]
        self.position             = int(values[2])
        status = int(values[3])
        if status == 1:
            self.status               = "NEEDED"
        elif status == 2:
            self.status               = "DONE"
        elif status == 4:
            self.status               = "FAILED"
        elif status == 8:
            self.status               = "ABORTED"
        elif status == 16:
            self.status               = "INPROGRESS"
        elif status == 32:
            self.status               = "TERMINATE"
        
    def part(self):
        return  "         being replicated to %s " % self.vault + " status: %s " % self.status 

    def __repr__(self):
        return  "backup_no: %5d" % self.backup_no +\
            "     being replicated to %s " % self.vault + " status:%s " % self.status 

class node:
    """psql command line return; so you have to digest it here"""
    def __init__(self, line):
        values = line.split("|")
        self.node_no          = int(values[0])
        self.node_name        = values[1].strip()
        self.type             = int(values[2])
        self.os               = NodeType(int(values[3]))
        self.status           = int(values[4])
        self.priority         = 999
        if len(values[5].strip()):
            self.priority     = int(values[5])
        self.options          = 0
        self.version          = values[7]
        self.properties       = 0
        if len(values[6].strip()):
            self.options      = int(values[6])
        if len(values[8].strip()):
            self.properties   = int(values[8])
        self.sync = False
        #for instance in application_instances:
        #    if instance.node_no != self.node_no: continue
        #    if instance.sync == "t":
        #        self.sync = True

        return

    def __repr__(self):
        return "node_no: %2d" % self.node_no + "     node_name: %15s  [%s] ++++ " % (self.node_name, self.os) + \
               "   version: %20s " % self.version + "  sync: %d" % self.sync


class notification:
    def __init__(self, line):
        values = line.split("|")
        self.notification_id      = int(values[0])
        self.category_id          = int(values[1])
        self.short_description    = values[2].strip()
        self.primary_text         = values[3].strip()
        self.severity             = int(values[4])
        self.state                = str2bool(values[5].strip())
        self.action               = values[6].strip()
        self.alert                = str2bool(values[7].strip())
        self.trap                 = str2bool(values[8].strip())
        self.email                = str2bool(values[9].strip())
        self.psa = False
        self.log = False
        if len(values) >11:
           self.psa                  = str2bool(values[10].strip())
           self.log                  = str2bool(values[11].strip())
        else:
           self.log                  = str2bool(values[10].strip())
                                       
        
    def __repr__(self):
        return
                                                                                            
class vmware_esx_server:
   def __init__(self, line):
      values = line.split("|")
      self.esx_uuid             = values[0]
      self.name                 = values[1]

      current = values[2].strip()
      self.current = False
      if current == 't':
         self.current           = True
         
      self.version              = values[3].strip()
      self.num_cpu_sockets      = int(values[4])
      
   def __repr__(self):
      return "%30s %25s %5s %8s    %d" % (self.esx_uuid, self.name, self.current, self.version, self.num_cpu_sockets)

class application_lookup:
   def __init__(self, line):
      values = line.split("|")
      self.app_id               = int(values[0])
      self.name                 = values[1].strip()
      self.type                 = values[2].strip()

   def __repr__(self):
      return "%d" % self.app_id +"%s" % self.name +"%s" % self.type
                                    

def get_trap_type(trap_type_id):
   trap_type = ""
   for tt in CONFIG.bpdb_trap_types:
      if trap_type_id == tt.trap_type_id:
         return tt.trap_type
   return trap_type



def get_db_entries(name, host='localhost'):
   objects = []
   command =""
   filename = "/var/log/monitoring_disks/bp.%s.dbout" % (name)
   
   if CONFIG.dbON:
      # default query
      command = "/usr/bin/psql bpdb postgres -tq -c \"SELECT * FROM bp.%s\" | grep -v ^$" % name
      # something else
      if name == "notifications":
         command = "/usr/bin/psql bpdb postgres -tq -c \"SELECT * FROM bp.%s WHERE" % name
         command+= "(notification_id>=137 and notification_id<=139) OR "
         command+= "(notification_id>=155 and notification_id<=157) OR"
         command+= "(notification_id=150) "
         command+= "ORDER BY notification_id\" | grep -v ^$" 
      ###   
      #elif name == "sds_stats":
      #   command = "/usr/bin/psql bpdb postgres -tq -c \"SELECT client_name,backup_no,sync_type,"
      #   command+= "destination,presync_seconds,sync_seconds,postsync_seconds,done_epoch,"
      #   command+= "bytes_on_dpu,bytes_syncd,bytes_on_dpv,message,stat_time FROM bp.%s ORDER BY stat_time \" | grep -v ^$" % name
      elif name == "application_instances":
         command = "/usr/bin/psql bpdb postgres -tq -c \"SELECT instance_id,node_no,app_id,current,"
         command+= "key1,key2,key3,key4,sync,priority,wvf,credential_id,app_aware,identity,discovery_time "
         command+= "FROM bp.%s ORDER BY node_no \" | grep -v ^$" % name
      elif name == "replication_queue" or name == "operation_progress":   
         command = "/usr/bin/psql bpdb postgres -tq -c \"SELECT * FROM bp.%s ORDER BY backup_no \" | grep -v ^$" % name
      elif name == "backup_reps":
         command = "/usr/bin/psql bpdb postgres -tq -c \"SELECT n.node_no,n.node_name,n.type,n.os,n.properties,"
         command+= "b.backup_no,b.start_time,b.elapsed_time,b.properties,b.total_netpaths,b.total_megs,b.type,b.instance_id "
         command+= "FROM bp.nodes AS n JOIN bp.backups AS b ON n.node_no=b.node_no "
         command+= "WHERE b.type!=1038 ORDER BY backup_no \" | grep -v ^$"
      elif name == "vmware_esx_servers":
         command = "/usr/bin/psql bpdb postgres -tq -c \"SELECT esx_uuid,name,current,version,num_cpu_sockets "
         command+= "FROM bp.%s \" | grep -v ^$" % name
      elif name == "application_lookup":
         command = "/usr/bin/psql bpdb postgres -tq -c \"SELECT app_id,name,type "
         command+= "FROM bp.%s \" | grep -v ^$" % name
      elif name == "nodes":
         command = "/usr/bin/psql bpdb postgres -tq -c \"SELECT node_no,node_name,type,os,status,priority,options,regexp_replace(version,E'[\\n\\r]+',' ','g'),"
         command+= "properties FROM bp.%s ORDER BY node_no\" | grep -v ^$" % name
      elif name == "backups":
         command = "/usr/bin/psql bpdb postgres -tq -c \"SELECT backup_no,node_no,start_time,elapsed_time,properties,total_netpaths,total_megs,status,type "
         command+= "FROM bp.%s ORDER BY backup_no\" | grep -v ^$" % name
      if name == "snmp_config" or name == "notifications": # make a result saved in the log
         if not os.path.exists("/var/log/monitoring_disks"):
            os.system("/bin/mkdir /var/log/monitoring_disks")
         os.system("%s > %s" % (command, filename) )
         
      #print command   
   else:
      if os.path.exists(filename):
         command = "/bin/cat %s" % filename
      else:
         return objects

   if CONFIG.debug:
      print "#%s:get_db_entries()"%basename
      print "     ", command

   lines = commands.getoutput(command).splitlines()
   for line in lines:
      if "categories" in name:
         object = category(line)
      elif "trap_types" in name:
         object = trap_type(line)
      elif "backups" in name:
         object = backup(line)
      elif "snmp_config" in name:
         object = snmp_config(line)
      elif "alerts" in name:
         object = alert(line)
      elif "backup_files" in name:
         object = backup_file(line)
      elif "notifications" in name:
         object = notification(line)
      ###
      elif name =="backup_reps":
         object = backup_rep(line)
      elif name =="nodes":
         object = node(line)
      elif name == "application_instances":
         object = application_instance(line)
      elif name == "sds_stats":
         object = sds_stat(line)
      elif name == "vmware_esx_servers":
         object = vmware_esx_server(line)
      elif name == "application_lookup":
         object = application_lookup(line)
      elif name == "replication_queue":
         object = replication_queue(line)
      elif name == "operation_progress":
         object = operation_progress(line)
         
      objects.append(object)

   return objects   

#
# Instead of reading from database, we want to do hardcoded information because of verseion independence #

#filename = "/var/log/monitoring_disks/categories.%s.dbout" % (versionDate)
#if not os.path.exists(filename):
#      os.system("/usr/bin/psql bpdb postgres -tq -c \"select * from bp.categories\" | grep -v ^$ > %s" % (filename) )
#CONFIG.bpdb_categories = get_db_entries(filename)
CONFIG.bpdb_categories = []
CONFIG.bpdb_categories.append(category( 1,  "Platform"))
CONFIG.bpdb_categories.append(category(15,  "Disk Health"))
CONFIG.bpdb_categories.append(category(16,  "IPMI Event"))
CONFIG.bpdb_categories.append(category(99,  "Test Trap"))
CONFIG.bpdb_categories.append(category(100, "Dynamic Trap"))

#filename = "/var/log/monitoring_disks/trap_types.%s.dbout" % (versionDate)
#if not os.path.exists(filename):
#      os.system("/usr/bin/psql bpdb postgres -tq -c \"select * from bp.trap_types\" | grep -v ^$ > %s" % (filename) )
#CONFIG.bpdb_trap_types = get_db_entries(filename)
CONFIG.bpdb_trap_types = []
CONFIG.bpdb_trap_types.append(trap_type(12,  "Version Information"))
CONFIG.bpdb_trap_types.append(trap_type(15,  "Disk health state has changed"))
CONFIG.bpdb_trap_types.append(trap_type(16,  "IPMI Event has occurred"))
CONFIG.bpdb_trap_types.append(trap_type(99,  "Test Trap"))
CONFIG.bpdb_trap_types.append(trap_type(100, "Dynamic Trap to collect platform data"))

CONFIG.bpdb_snmp_config   = []
CONFIG.bpdb_notifications = []

if CONFIG.majorVersion >= 5:
   CONFIG.bpdb_snmp_config   = get_db_entries("snmp_config")
   CONFIG.bpdb_notifications = get_db_entries("notifications")
