import os,sys   
abs_path = os.path.abspath(sys.argv[0])
basename = os.path.basename(sys.argv[0])
dirname = abs_path.replace(basename, "")

if not "%s/diskmon/lib" % dirname in sys.path:
   sys.path.insert(0,"%s/diskmon/lib" % dirname) # always with regard to parent script
if not "/usr/bp/lib/monitoring" in sys.path:
   sys.path.append("/usr/bp/lib/monitoring")

from device    import *
from support_message import *
class SMART: # struct for Disk class
   pass

#################class Disk ##########################
class Disk:  # inheritance: Disk_LSI, Disk_3ware, Disk_OS
   """Holding partition information from smartctl""" 
   # itwa is only for 3ware/AMCC card
   def __init__(self, appliance, device, status="", lsscsi_model=""): #encId is only for LSI drive
      self.hwraid_opt,self.suffix,self.statusInitial,self.progress ="","","",""
      self.origDevice = device
      self.device     = device
      self.status     = status
      self.appliance  = appliance
      self.loghead    ='  ==='
      self.logmid     ='               Drive Either Not Existing Or Not Recognized'
      self.logend     =''
      self.lsscsi_model=lsscsi_model
      self.extra      =''
      if self.status == "Failed":
         self.statusInitial = "F"
      elif self.status == "Dead":
         self.statusInitial = "DD"
      elif self.status == "Offline":
         self.statusInitial = "OFF"
      elif "Unconfigured(bad)" in self.status:
         self.statusInitial = "UB"
      elif "Unconfigured(good)" in self.status: # status="Unconfigured(good),"
         self.statusInitial = "UG"
      elif self.status == "Rebuild" or "REBUILD" in self.status or "RBLD" in self.status: # or "REBUILD" in self.status:
         self.statusInitial = "R"
      elif "DEVICE-ERROR" in self.status:
         self.statusInitial = "DE"
      elif "ECC-ERROR" in self.status:
         self.statusInitial = "EE"
      elif "NOT-PRESENT" in self.status: # should be above DEGRAD due to NOT-PRESENT_DEGRADE in status
         self.statusInitial = "NP"
      elif "DEGRAD" in self.status:  #There is DEGRADED-INIT
         self.statusInitial = "DG"
      elif "CFG" in self.status:
         self.statusInitial = "CF"
      elif "TBD" == self.status:
         self.statusInitial = "TBD"
      else:   
         self.statusInitial = "o"

      ### SMART Information
      self.smart = SMART()   

      ### Trap Related
      self.trapTargAdd =""
      self.trapMessAdd =""
      self.smart_for_trap=""

      
   ############################         
   def loc_subpartition(self, smart_num):
      for spart in self.part:
         if smart_num >= spart.start and smart_num <= spart.end:
            return spart.id
      return -1

   ############################
   def PrepLog(self,addendum=""):   
      log = "%2d" % self.model.thrput
      if self.model.sectorSize==4096:
         log = log + "AF"
      else:
         log = log + "=="

      self.loghead = log   
      self.logmid = "=" # this is for X: archiving

      # Logic: For trap, we want to make sure if pending sector is persistent at least 12 hours
      #        When customer support run the script, we want to show it as it is.
      log = ""
      theThreshold_maxPendingSector = CONFIG.maxPendingSectorAF
      if self.model.sectorSize==512:
         theThreshold_maxPendingSector = CONFIG.maxPendingSector512

      if self.smart.pending > theThreshold_maxPendingSector or self.smart.uncorr > theThreshold_maxPendingSector:
         log = log + "*"
         #oneLine = get_printout("/usr/bin/tail -1 %s/%s.%s.dat" % (CONFIG.directory_paths[CONFIG.ibase], self.smart.modelName, self.smart.serialNumber))[0]
         #oneLineWords = oneLine.split()
         #hourspan = int(oneLineWords[2]) - int(oneLineWords[1])
         #if int(oneLineWords[3])>0 and hourspan >= CONFIG.maxHourSpanForPendingSector:   
         
      if self.rscDrive:     log += "x"   
      if self.serDrive:         log += "s" 

      # notified by Luis 8/8/2014         
      #0===   o       OS    /dev/sdb         -1 (   -1 d)    -9    -9    -9  000/100     0      -9/    -9           ST_M13FQBL                 QNR_BFW   01101130      0 GB 0xFF
      if self.smart.hours<0 and not self.statusInitial in ["UB", "UG"]: #Unconfigured(Bad) sometimes can't access SMART information
         self.statusInitial = "F"

      if ("*" in log or "x" in log or "s" in log) and "o" in self.statusInitial:
         log += ""
      else:   
         log += self.statusInitial

      #if self.statusInitial == "o" and len(log):
      #   self.statusInitial = log         
         
      if len(log) <5:
         for n in range(0, 5-len(log)):
            log += " "
      extra,theDevice = "",self.device

      if self.smart.unknown ==1:
         log = "==== ???" + "  >>>> smartctl version is out-of-date and could not retrieve information <<<<"
      else:
         if self.appliance.controller == "LSI":
            if isinstance(self, Disk_LSI):    
               if not len(extra):
                  extra = "Bay#%s" % addendum
               if self.model.capacity == 1000 or self.SAS:
                  extra = "OS_HWR"
                  if self.appliance.name=="943":
                     extra = "DBINTN"
               if ("SSD" in self.model.model or "Crucial" in self.model.model or "Micron" in self.model.model) and self.appliance.cachecade:
                  extra = "CACHECD"
                  if not self.IsCacheCadeSetup:
                     extra = "?CACHECD"
                  theDevice=""
            elif len(addendum)==0:
               extra = "OSdriv"
               if isinstance(self, Disk_OS):
                  if self.archive:
                     extra = "Archve"
                  elif self.appliance.flashcache:  # possible flashcache instead of Cachecade
                     if not self.rotational:
                        extra ="?CACHE"
            modelName=self.smart.modelName
            serialNumber=self.smart.serialNumber

            if not len(modelName):
               modelName="*%s*"%self.smart.ex_modelName
               serialNumber="*%s*"%self.smart.ex_serialNumber
               
            log += "%9s%12s" % (extra, theDevice) + "%3s " % self.suffix + "  %5d " % self.smart.hours + "(%5d d) " % self.smart.days \
                  + "%5d " % self.smart.pending + "%5d " % self.smart.uncorr + "%5d" % self.smart.realloc \
                  + "  %03d/%03d" % (self.smart.seek_error_rate_scr,self.smart.seek_error_rate_worst)\
                  + "  %4d  %6d/%6d" % (self.nAtaErrors, self.smart.total_tb_written, self.smart.total_tb_read)\
                  + "%30s%24s  %9s %6s GB" % (modelName, serialNumber, self.smart.firmwareVersion, self.model.capacity)

            if isinstance(self, Disk_LSI):
               log += "   %8s" % self.braket

         else:  # 3ware or OS
            unitname = ""
            if isinstance(self, Disk_3ware):
               unitname = self.unitname
            else:
               unitname = "OS"
               if self.appliance.showBayNumbersIn1U:
                  if not self.archive:
                     unitname = "BAY#%d" % self.bayNumber
               
               if isinstance(self, Disk_OS):
                  if self.archive:
                     unitname = "EX"
                  if self.flashcache:
                     unitname = "CACHE"
                  else:
                     if not self.rotational:
                        unitname ="?CACHE"
                     
            log += "%6s%12s" % (unitname,self.device) + "%3s " % self.suffix + "  %5d " % self.smart.hours + "(%5d d) " % self.smart.days \
                + "%5d " % self.smart.pending + "%5d " % self.smart.uncorr + "%5d" % self.smart.realloc \
                + "  %03d/%03d" % (self.smart.seek_error_rate_scr,self.smart.seek_error_rate_worst) \
                + "  %4d  %6d/%6d" % (self.nAtaErrors, self.smart.total_tb_written, self.smart.total_tb_read)\
                + "%30s%24s  %9s %6s GB" % (self.smart.modelName, self.smart.serialNumber, self.smart.firmwareVersion, self.model.capacity)

            #####
         log += " 0x%s" %self.model.hex      
         if CONFIG.detailSMART or CONFIG.runDirname == "/usr/bp/bin/":
            log += " %2d  %4d %10d  %4d" %(self.smart.tempert, self.smart.seek_error_rate_num, self.smart.seek_error_rate_deno, self.smart.raw_read_error_rate_num)
            log += " %10d  %12d %12d" %(self.smart.raw_read_error_rate_deno, self.smart.total_mb_written, self.smart.total_mb_read)
            log += " %3d  %3d  %3d" % (self.smart.power_cycle_count, self.smart.reported_uncorrect, self.smart.command_timeout)
            
      self.logend =log 
      self.extra  =extra

      self.speclog  = "  %5d   %5d %5d   %5d" % (self.smart.hours,   self.smart.pending, self.smart.uncorr, self.smart.realloc)
      self.speclog += "  %3d   %5d %5d"       % (self.smart.tempert, self.smart.seek_error_rate_num, self.smart.raw_read_error_rate_num) 
      self.speclog += "  %3d %3d %3d" % (self.smart.spin_retry_count,   self.smart.end_to_end_error,   self.high_fly_writes)
      self.speclog += "  %3d %3d %3d" % (self.power_off_retract_count,  self.load_cycle_count, self.smart.calibration_retry_count)     
      self.speclog += "  %3d %3d   %5d  %12d"  % (self.smart.power_cycle_count, self.smart.reported_uncorrect, self.smart.g_sense_error_rate, self.smart.command_timeout)


      self.smart_for_trap= "%dd R%d P%d S%d 0x%s SN%s W%d)" % (self.smart.days, self.smart.realloc, self.smart.pending, self.smart.seek_error_rate_worst, self.model.hex, self.smart.serialNumber, self.smart.total_tb_written)

      
   def FillSMART(self, isSASDrive=False):
      # get partition
      if CONFIG.debug:
         print "%s: retrieving smart information is in progress" % (self.device+self.suffix)

      self.smart.hfw = -99 # some disks (512 Hitachi) don't have this info
      self.smart.airflow, self.smart.airflowscr, self.smart.airflowthr =  0,  0,  0
      self.smart.realloc, self.smart.reallocscr, self.smart.reallocthr = -9, -9, -9
      self.smart.pending, self.smart.uncorr, self.smart.unknown  = -9, -9, 0
      self.smart.days,    self.smart.hours, self.smart.tempert, self.smart.ecc= -1, -1, -1, -99
      self.smart.seek_error_rate_deno, self.smart.seek_error_rate_num, self.smart.seek_error_rate_raw =0,0,0
      self.smart.seek_error_rate_scr , self.smart.seek_error_rate_thr, self.smart.seek_error_rate_worst =0,0,100
      self.smart.raw_read_error_rate_deno, self.smart.raw_read_error_rate_num, self.smart.raw_read_error_rate_raw =0,0,0
      self.smart.raw_read_error_rate_scr,  self.smart.raw_read_error_rate_thr, self.smart.raw_read_error_rate_worst=0,0,100
      self.smart.total_lbas_written, self.smart.total_lbas_read = -9, -9
      self.smart.total_tb_written,   self.smart.total_tb_read = -9, -9
      self.smart.total_mb_written,   self.smart.total_mb_read = -9, -9
      self.smart.power_cycle_count,  self.smart.reported_uncorrect, self.smart.command_timeout=0,0,0
      self.smart.spin_retry_count,   self.smart.end_to_end_error,   self.high_fly_writes=0,0,0
      self.smart.g_sense_error_rate, self.power_off_retract_count,  self.load_cycle_count, self.smart.calibration_retry_count=0,0,0,0 
      self.smart.ex_modelName, self.smart.ex_serialNumber='','' 
      self.rscDrive, self.serDrive, self.pndDrive = False, False, False
      modelName, serialNumber, firmwareVersion = "","",""
      self.diskErrors = []
      self.nAtaErrors = 0

      command="%s -i %s" % (CONFIG.smartctl_Path, str(self.device)) + " %s| grep ':' | grep -v http" % self.hwraid_opt
      lines =get_printout(command)
      noDSATOption=False
      if "Failed" in lines[0] and isinstance(self, Disk_OS):
         command="%s -i %s" % (CONFIG.smartctl_Path, str(self.device)) + " | grep ':' | grep -v http"
         noDSATOption=True
         lines =get_printout(command)

      for line in lines:
         words = line.split()
         if len(words)<2: continue
         if not isSASDrive and "SAS" in line:
            isSASDrive=True
         if "Model:"  in words[1]:
            modelName = line.split(":")[1].strip()
            modelName = modelName.replace(" ", "_")
         elif "Number:"  in words[1]:
            serialNumber= words[len(words)-1]
         elif "Firmware"  in words[0]:
            firmwareVersion= words[len(words)-1]
            break

      if isSASDrive:
         for line in lines:
            words = line.split()
            if len(words)<2: continue
            if "Product:"  in words[0]:
               modelName = line.split(":")[1].strip()
               modelName = modelName.replace(" ", "_")
            elif "Revision:"  in words[0]:
               firmwareVersion= words[len(words)-1]
            elif "number:" in words[1]:
               serialNumber= words[len(words)-1]
               serialNumber= serialNumber[0:8]
               break

      self.smart.modelName, self.smart.serialNumber, self.smart.firmwareVersion = modelName, serialNumber, firmwareVersion
      if "disk" in CONFIG.verbose:
         print "disk.py:: %20s "% self.smart.modelName, self.smart.serialNumber, self.smart.firmwareVersion

      # For failed drive 
      if not len(modelName):
         self.model = Device(255, "NotFound", "NotFound", 0,0,7200,512)   
         return

      # Exceptional Case..
      #[root@pny-recovery-822-01 ~]# smartctl -i /dev/sdd
      #=== START OF INFORMATION SECTION ===
      #Device Model:     [No Information Found]
      #Serial Number:    [No Information Found]
      #SMART support is: Ambiguous - ATA IDENTIFY DEVICE words 82-83 don't show if SMART supported.
      #[root@pny-recovery-822-01 ~]# lsscsi | grep sdd
      #[7:0:0:0]    disk    ATA      WDC WD40EZRX-00S 80.0  /dev/sdd
      
      if "[No_Information_Found]" in self.smart.modelName:
         self.model = Device(255, self.lsscsi_model, "NotSupported", 0,0,0,0)
         self.smart.modelName=self.lsscsi_model # output is per self.smart.modelName
         return
            
      self.model = self.getModelDetail(modelName)      

      if isSASDrive:
         self.FillSMART_SAS()
         return
      else:
         self.FillSMART_SATA(noDSATOption)
         
   #########################
   # Available attributes in SAS drive SMART
   # (1) Temperature
   # (2) Number of defect list ==> nAtaErrors
   # (3) Power_on_hours, days
   # (4) Non-medium error count ==> seek_error_rate_raw
   # (5) SMART Health Status ==> seek_error_rate_scr
   # (6) read /write ==> self.smart.total_tb_read, self.smart.total_tb_written
   # (7) nDelayed ==> pending
   # (8) nUncorr  ==> uncorr
   # (9) nCorr/1M    ==> realloc 
   ##############################################

   def FillSMART_SAS(self):
      command="%s -a %s %s | grep  -v smart | grep -v ^$" % (CONFIG.smartctl_Path, self.device, self.hwraid_opt)
      nDelayed=0
      nCorrected=0
      nUncorrected=0
     
      for line in get_printout(command):
         words = line.split()
         if "Current Drive Temperature:" in line:
            self.smart.tempert=int(words[3])
         elif "Elements in grown defect list:" in line:
            self.nAtaErrors=int(words[5])
         elif "number of hours powered up" in line:
            self.smart.hours =int(float(words[6]))
            self.smart.days  =int(self.smart.hours/24.)
         elif "Non-medium error count:" in line:
            self.smart.seek_error_rate_raw=int(words[3])
         elif "SMART Health Status:" in line:
            if words[3].strip()=="OK":
               self.smart.seek_error_rate_scr=100
            else:
               self.smart.seek_error_rate_scr=0
         elif "read:" in words[0]: 
            nDelayed+=int(words[2])
            nCorrected+=int(words[4])
            nUncorrected+=int(words[7])
            self.smart.total_tb_read=int(float(words[6])/1024)            
            self.smart.total_mb_read=int(float(words[6])*1024)            
         elif "write:" in words[0]: 
            nDelayed+=int(words[2])
            nCorrected+=int(words[4])
            nUncorrected+=int(words[7])
            self.smart.total_tb_written=int(float(words[6])/1024)            
            self.smart.total_mb_written=int(float(words[6])*1024)            
      self.smart.pending=nDelayed
      self.smart.uncorr=nUncorrected
      self.smart.realloc=int(nCorrected/1000000)         

   def FillSMART_SATA(self, noDSATOption):

      command="%s -A %s " % (CONFIG.smartctl_Path, self.device)
      if not noDSATOption: command += self.hwraid_opt
      command+=" 2>&1 | grep x | grep -v smart"

      for line in get_printout(command):
         wordsFromline = line.split()
         idAttr = int(wordsFromline[0])
         attrName = wordsFromline[1]
         rawValue = wordsFromline[9]
         if "Reallocated_Sec"   in attrName:     #5 Reallocated_Sector_Ct
            self.smart.realloc    = int(rawValue)
            self.smart.reallocscr = int(wordsFromline[3])
            if not '--' in wordsFromline[5]:   #WD black 4TB return ---
               self.smart.reallocthr = int(wordsFromline[5])
         elif "Reallocate_NAND_Blk_Cnt"   in attrName:     #5 Reallocate_NAND_Blk_Cnt in Crucial SSD
            self.smart.realloc    = int(rawValue)
            self.smart.reallocscr = int(wordsFromline[3])
         elif "Power_On_Ho" in attrName:
            if rawValue.isdigit():
               self.smart.hours   = int(rawValue)
            else: # case of SSD  # 943-200-50027  -894850+1200
               self.smart.hours   = int(rawValue.split('h')[0])-894850-408 #897002h+51m+50.830s  # found when I install SSD
               if self.smart.firmwareVersion=="520i":
                  self.smart.hours += 893983 +842
               
            self.smart.days    = float(self.smart.hours)/24  
         elif "Seek_Error_" in attrName:
            self.smart.seek_error_rate_raw = int(rawValue)
            self.smart.seek_error_rate_scr = int(wordsFromline[3])
            if not '--' in wordsFromline[5]: #WD black 4TB return ---
               self.smart.seek_error_rate_thr = int(wordsFromline[5])
            self.smart.seek_error_rate_worst = int(wordsFromline[4])

         elif "Raw_Read_Error_" in attrName:
            self.smart.raw_read_error_rate_raw = int(rawValue)
            self.smart.raw_read_error_rate_scr = int(wordsFromline[3])
            if not '--' in wordsFromline[5]: #WD black 4TB return ---
               self.smart.raw_read_error_rate_thr = int(wordsFromline[5])
            self.smart.raw_read_error_rate_worst = int(wordsFromline[4])
            
         elif "Current_Pen" in attrName: self.smart.pending = int(rawValue)
         elif "Offline_Unc" in attrName: self.smart.uncorr  = int(rawValue)
         elif "Airflow_Tem" in attrName:
            self.smart.airflow = int(rawValue)
            self.smart.airflowscr = int(wordsFromline[3])
            if not '--' in wordsFromline[5]: #WD black 4TB return ---
               self.smart.airflowthr = int(wordsFromline[5])
         elif "ure_Celsius" in attrName: self.smart.tempert = int(rawValue)
         elif "High_Fly_Wr" in attrName: self.smart.hfw = int(rawValue)
         elif "Hardware_EC" in attrName: self.smart.ecc = int(rawValue)
         elif "Media_Wearout" in attrName:  #Intel SSD
            self.smart.seek_error_rate_scr = int(wordsFromline[3])
            self.smart.seek_error_rate_worst = int(wordsFromline[4])
         elif "Percent_Lifetime_Used" in attrName or idAttr==202:  #Crucial SSD
            self.smart.seek_error_rate_scr = int(wordsFromline[3])
            self.smart.seek_error_rate_worst = int(wordsFromline[4])
         #elif "Available_Reservd_Space" in attrName:        #Intel SSD
         #   self.smart.seek_error_rate_raw = int(rawValue)
         #   self.smart.seek_error_rate_scr = int(wordsFromline[3])
         #   self.smart.seek_error_rate_thr = int(wordsFromline[5])
         #   self.smart.seek_error_rate_worst = int(wordsFromline[4])
         elif "End-to-End_Error" in attrName:   #Intel SSD 
            self.smart.pending = int(rawValue)
         elif "Erase_Fail_Count" in attrName: #Intel SSD 
            self.smart.uncorr  = int(rawValue)
         elif idAttr==241:
            self.smart.total_lbas_written = int(rawValue)
            #if "Total_LBAs_Written" in attrName:  # make it default for Unknown Attribute  
            self.smart.total_tb_written = float(self.smart.total_lbas_written)*0.5/1024/1024/1024
            self.smart.total_mb_written = float(self.smart.total_lbas_written)*0.5/1024
            if "Host_Writes_32MiB" in attrName:
               self.smart.total_tb_written = float(self.smart.total_lbas_written)/32/1024
               self.smart.total_mb_written = float(self.smart.total_lbas_written)*1024/32
         elif idAttr==242:
            self.smart.total_lbas_read = int(rawValue)
            #if "Total_LBAs_Read" in attrName:  # make it default for Unknown Attribute
            self.smart.total_tb_read = float(self.smart.total_lbas_read)*0.5/1024/1024/1024
            self.smart.total_mb_read = float(self.smart.total_lbas_read)*0.5/1024
            if "Host_Reads_32MiB" in attrName:
               self.smart.total_tb_read = float(self.smart.total_lbas_read)/32/1024
               self.smart.total_mb_read = float(self.smart.total_lbas_read)*1024/32
         elif idAttr==246:
            self.smart.total_lbas_written = int(rawValue)
            self.smart.total_tb_written = float(self.smart.total_lbas_written)*0.5/1024/1024/1024
            self.smart.total_mb_written = float(self.smart.total_lbas_written)*0.5/1024
         elif idAttr==10:
            self.smart.spin_retry_count =int(rawValue)
         elif idAttr==11:
            self.smart.calibration_retry_count =int(rawValue)
         elif idAttr==12:
            self.smart.power_cycle_count =int(rawValue)
         elif idAttr==184:
            self.smart.end_to_end_error=int(rawValue)
         elif idAttr==187:
            self.smart.reported_uncorrect=int(rawValue)
         elif idAttr==188:
            self.smart.command_timeout=int(rawValue)
         elif idAttr==189:
            self.smart.high_fly_writes=int(rawValue)
         elif idAttr==191:
            self.smart.g_sense_error_rate=int(rawValue)
         elif idAttr==192:
            self.smart.power_off_retract_count=int(rawValue)
         elif idAttr==193:
            self.smart.load_cycle_count=int(rawValue)

      if CONFIG.confirm and CONFIG.selfhealing:
         if not self.model.WTB:
            self.smart.total_tb_written, self.smart.total_tb_read  = -9, -9

      
      ##### ATA error diagnostic
      lineCount, errorId, errorHour, errorName =-99,0,-9,''
      detailedErrors, diskErrors = [],[] 
      #Sometimes, drive becomes dead so that it should be excluded by grep -v:
      #Error SMART Values Read failed: megasas_cmd result: 0.14 = 0/46
      #Smartctl: SMART Read Values failed.
      for line in get_printout("%s -l error %s %s" % (CONFIG.smartctl_Path, self.device, self.hwraid_opt)):
         if "Error SMART Values Read failed"     in line: break
         if "Error SMART Thresholds Read failed" in line: break 
         if "Error SMART Error Log Read failed"  in line: break

         words = line.split()
         if not len(words): continue
         if words[0] == 'Error':
            if lineCount>0:
               diskErrors.append(DiskError(errorId, errorHour, errorName, detailedErrors))
             
            lineCount =0
            errorId =int(words[1])
            errorHour = int(words[7])
            errorName = ''
            detailedErrors = []
         else:
            if lineCount == 5:
               if len(words) >8 :
                  errorName = words[8]
            elif lineCount >8:
               errCtd = ''  #passing split words into one string 
               for i in range(len(words)-9):
                  errCtd += "%s " % words[9+i]
               errCtd = errCtd.strip()    # to get rid of the last whitespace
               detailedErrors.append(errCtd)
         lineCount +=1    
         
      diskErrors.append(DiskError(errorId, errorHour, errorName, detailedErrors))
      diskErrors.sort(key=lambda DiskError: DiskError.Id)
      self.diskErrors = diskErrors
      if errorId:
         self.nAtaErrors = errorId-1+len(diskErrors)

      if self.smart.modelName[0:2] == "ST":
         self.smart.seek_error_rate_deno = toHex(self.smart.seek_error_rate_raw)
         if len(self.smart.seek_error_rate_deno) <= 8:
            self.smart.seek_error_rate_deno = int("0x%s"%self.smart.seek_error_rate_deno, 16)
         else:
            nchar = len(self.smart.seek_error_rate_deno)
            self.smart.seek_error_rate_num  = int("0x%s"%self.smart.seek_error_rate_deno[0:(nchar-8)], 16)
            self.smart.seek_error_rate_deno = int("0x%s"%self.smart.seek_error_rate_deno[-8:], 16)

         self.smart.raw_read_error_rate_deno = toHex(self.smart.raw_read_error_rate_raw)
         if len(self.smart.raw_read_error_rate_deno) <= 8:
            self.smart.raw_read_error_rate_deno = int("0x%s"%self.smart.raw_read_error_rate_deno, 16)
         else:
            nchar = len(self.smart.raw_read_error_rate_deno)
            self.smart.raw_read_error_rate_num  = int("0x%s"%self.smart.raw_read_error_rate_deno[0:(nchar-8)], 16)
            self.smart.raw_read_error_rate_deno = int("0x%s"%self.smart.raw_read_error_rate_deno[-8:], 16)
         
      elif self.smart.modelName[0:2] == "WD":   
         self.smart.seek_error_rate_num = int(self.smart.seek_error_rate_raw)

      if CONFIG.testOn=="0": 
         if isinstance(self, Disk_3ware):
            if self.portNo == "p1":  #TEST
               self.smart.realloc =5002
            if self.portNo == "p5":  #TEST
               self.smart.pending =8
               self.smart.realloc =9008
            if self.portNo == "p7":  #TEST
               self.smart.seek_error_rate_scr =28

      if CONFIG.testOn=="01" or CONFIG.testOn=="02": 
         if isinstance(self, Disk_3ware):
            if self.portNo == "p5":  #TEST
               self.smart.pending =8

      if CONFIG.testOn:         
         if "sd" in CONFIG.testOn:
            if isinstance(self, Disk_OS):
               if CONFIG.testOn in self.device:
                  self.smart.realloc =9008

      if CONFIG.testOn =="2":
         if isinstance(self, Disk_LSI):
            if "sda" in self.device and "23" in self.suffix:
               self.smart.realloc =20000

      if self.model.sectorSize==512:
         if "SSD" in self.model.model and self.smart.realloc > CONFIG.maxMediaWearout :
            self.rscDrive = True
         if self.smart.realloc >CONFIG.maxReallocNumber512:
            self.rscDrive = True
               
      elif self.model.sectorSize==4096:
         if self.smart.realloc >CONFIG.maxReallocNumberAF:
            self.rscDrive = True

      if int(self.smart.seek_error_rate_worst) < CONFIG.minSERScore and int(self.smart.seek_error_rate_worst)>0:
         self.serDrive = True

      theThreshold_maxPendingSector = CONFIG.maxPendingSectorAF
      if self.model.sectorSize==512:
         theThreshold_maxPendingSector = CONFIG.maxPendingSector512
         
      self.pndDrive = self.smart.pending > theThreshold_maxPendingSector






   def check_log_selftest(self):
      
      list_of_all_the_lines = get_printout("%s --log=selftest %s | egrep 'Complete|progress'" % (CONFIG.smartctl_Path,str(self.device)) )
      
      if len(list_of_all_the_lines)==0:
         #if CONFIG.debug: print "==== check_log_selftest(): 0"
         return 0
   
      for line in list_of_all_the_lines:
         # sometimes short test has been run
         wordsFromline = line.split()
         (smart_num, type, status, LBA_of_first_error) = (wordsFromline[1], wordsFromline[2], wordsFromline[4], wordsFromline[9])
         break

      ret_value = -99

      if status == 'Self-test':
         ret_value = 2
      elif status == 'Completed':
         ret_value = 1
      elif status == 'Completed:' :
         if LBA_of_first_error != '-':
            ret_value = int(LBA_of_first_error)
            
      if type == 'Short':
         ret_value *= -1
      #if CONFIG.debug: print "==== check_log_selftest(): %d" % ret_value
      return ret_value 


#   def printout(self):
#      print self.loghead + self.logmid + "   " + self.logend
   def __repr__(self):
      return self.loghead + self.logmid + "   " + self.logend

   def getModelDetail(self,modelName):
      for model in list_of_device():
         if modelName == model.model:
            return model
         
      return Device(255, "NotFound", "NotFound", 0,0,7200,512)   
##########################   
class Disk_LSI(Disk):
   def __init__(self, appliance, device, deviceId=-1, slotNo=-1, status="", encId=-1, vdname="", \
                   isVDCached=False, IsCacheCadeSetup=False, raidIdx=-9, pdType='SATA',\
                   diskGroup=-9, span=-9, arm=-9):#, diskgroup=-1, span=-1, arm=-1): #encId is only for LSI drive
      Disk.__init__(self, appliance, device, status)

      self.name  = "LSI"
      self.device=device
      self.VD_Id = -9
      self.vdname = vdname
      if vdname[-1:].isdigit():
         self.VD_Id = int(vdname[-1:])
      self.isVDCached=isVDCached
      self.IsCacheCadeSetup=IsCacheCadeSetup
      self.diskPosition=(diskGroup, span, arm)
      self.arrayNo=diskGroup+span
      self.rowNo=arm

      self.encId, self.slotNo = encId, slotNo
      self.suffix = ",%02d"%deviceId
      self.hwraid_opt = "-d sat+megaraid,%d" % deviceId
      self.SAS=False
      if pdType=="SAS":
         self.SAS, self.hwraid_opt = True, "-d megaraid,%d" % deviceId
      self.bayNo, self.braket =-1,""

      Disk.FillSMART(self, self.SAS) # filling self.smart as well as self.model

      if self.model.index  == 255:   return

      if self.statusInitial == "F" and self.smart.realloc >= 0:
         self.statusInitial = "FS"
      if "CacheCade" in vdname or raidIdx<0:  self.device=""  # For unassociated drive
      
      #Disk.PrepLog(self)   ###??? This is called again in disk_monitoring.py GetDiskDetails
      #self.PrepTrap()
      return
   
   def PrepTrap(self):
      if "WDC_WD1000CHTZ" in self.model.model:
         self.trapTargAdd =" S%02d" % int(self.slotNo)
         self.trapMessAdd =" [SLT%02d]" % int(self.slotNo)
      elif len(self.vdname):  # to exclude Cachecade
         self.trapTargAdd =" B%02d" % int(self.bayNo)
         self.trapMessAdd =" [Bay%02d]" % int(self.bayNo)
      return
   
   def SetDetailsForMatrix(self, bayNo, isNonBayDrive=False, nonBayNo=0): #bayNo is integer
      self.bayNo = bayNo
      #self.arrayNo = int(self.bayNo / 18)
      #self.rowNo = self.bayNo % 18
      self.nameForLogger = "bay%d" % bayNo
      if isNonBayDrive:
         self.nameForLogger = "nonbay%d" % nonBayNo
         
      #if self.encId == "N/A":
      #   self.arrayNo = 2
         
      if self.statusInitial == "DD" or self.statusInitial == "F": #"Dead" drive
         self.braket = "[--:--]"
         if len(self.encId):
            if self.encId == "N/A":
               self.braket = "[:%s]" % self.slotNo
            else:   
               self.braket = "[%s:" % self.encId + "%s]" % self.slotNo
      else:
         if self.encId == "N/A":
            self.braket = "[:%s]" % self.slotNo
         else:   
            self.braket = "[%s:" % self.encId + "%s]" % self.slotNo
             
      theWord = ""
      if self.bayNo >= 0:
         theWord = "%02d" %self.bayNo + self.braket + "%2s" %self.suffix[1:]
      else: 
         theWord = self.braket

      if len(theWord) <11 :
         for i in range(0,11-len(theWord)):
            theWord = theWord + " "
               
      self.word = theWord      
      self.PrepTrap()
      if self.status=="Dead" and os.path.exists(CONFIG.configFile):
         import ConfigParser
         diskConfig = ConfigParser.ConfigParser()
         diskConfig.read(CONFIG.configFile)
         if diskConfig.has_section(self.nameForLogger):
            self.smart.ex_modelName="%s"%diskConfig.get(self.nameForLogger,"Model")
            self.smart.ex_serialNumber="%s"%diskConfig.get(self.nameForLogger,"Serial")
      Disk.PrepLog(self) 
      return
   
   def SelfHeal(self, trapMessage):  #Disk_LSI
    
      arrayNo,rowNo=-9,-9
      if self.statusInitial in ("UB","UG") and not '?CACHE' in self.extra:
         for pdMissing in get_printout("%s -pdgetmissing -a0 -NoLog | grep -v -E -e 'Adapter|No|Exit|^[[:space:]]*$'" % CONFIG.MegaCli64_Path):
            #    0     0       1     1907200 MB
            #    1     2       16     3814912 MB
            words =pdMissing.split()
            array,row,size =int(words[1]), int(words[2]), int(words[3])/1024.
            import math
            if math.fabs(size-self.model.capacity/1.024/1.024/1.024)<20.:
               arrayNo,rowNo=array,row
      showMessage(self.name, self.statusInitial, self.braket, arrayNo, rowNo, self.extra)
      #if not CONFIG.selfhealing: return

      if self.statusInitial == "DD":
         trapMessage += "(**Needs Review** Model:%s SN:%s %s)" % (self.smart.ex_modelName, self.smart.ex_serialNumber,self.statusInitial) 
      elif self.statusInitial == "F":   #Failed and Nosmart avaiable
         trapMessage += "(**Needs Review** Model:%s SN:%s %s)" % (self.smart.ex_modelName, self.smart.ex_serialNumber,self.statusInitial) 
      elif self.statusInitial == "R": 
         if "Rebuild" in get_first_word("%s -pdrbld -showprog -physdrv%s -a0 -NoLog | grep Encl" % (CONFIG.MegaCli64_Path, self.braket)):
            trapMessage += "(Rebuild Progress: %s)" % self.smart_for_trap
      else: #If it's doable
         #TODO: at some point you want to avoid any rebuild on rscDrive
         #if self.rscDrive:   
         #else:
         # alert_id |                                                         alert_text                                                          | source_id |    target     | severity |  created   |  updated   | closed | notification_id
         # ----------+-----------------------------------------------------------------------------------------------------------------------------+-----------+---------------+----------+------------+------------+--------+-----------------
         # 176 | Virtual Drive: 0 ++++ Partially Degraded due to disk(s): [Bay23](**Needs Review** Rebuild Failed UB: 36d R0 P0)          |        15 | VD0 B23UB     |        1 | 1363974977 | 1363975108 | t      |             137
         # 177 | Virtual Drive: 0 ++++ Partially Degraded due to disk(s): [Bay23](Self-healing script initiated array rebuild UB: 36d R0 P0) |        15 | VD0 B23UB     |        3 | 1363975108 | 1363975108 | f      |             138
         if self.statusInitial == "FS" or self.statusInitial == "UB" or self.statusInitial == "UG" or self.statusInitial == "OFF":   
            CloseAnyActiveAlerts("B%s" % self.bayNo, 138)  # CONFIG.selfhealing
         


         if CONFIG.selfhealing and CONFIG.confirm: # FILTER IS NEEDED TO AVOID OVER AND OVER TRY
            if self.statusInitial == "OFF":
               #print "[ALERT] Drive with degraded drive (offline) found and marked as -OFF "
               #print "        ==> Useful command:   MegaCli64 -pdonline -physdrv%s -a0 -NoLog                      # to make sure if it's offline" % self.braket
               run_command("%s -pdonline -physdrv%s -a0 -NoLog" % (CONFIG.MegaCli64_Path, self.braket), 3)
            else:   
               if not os.path.exists("%s/%s.%s.SelfHealingAttempted" % (CONFIG.directory_paths[CONFIG.ibase], self.smart.modelName, self.smart.serialNumber)):
                  if self.statusInitial == "FS":   
                     run_command("%s -pdrbld -start -physdrv%s -a0 -NoLog" % (CONFIG.MegaCli64_Path, self.braket), 3) # sleep 3
      
                  elif self.statusInitial == "UB" and not '?CACHE' in self.extra:   
                     run_command("%s -pdmakegood -physdrv%s -a0 -NoLog" % (CONFIG.MegaCli64_Path, self.braket), 3) 
                     run_command("%s -cfgforeign -clear -a0 -NoLog" % CONFIG.MegaCli64_Path, 3)
                     # %s -pdgetmissing -a0 -NoLog run already above for the purpose of showMessage
                     run_command("%s -pdreplacemissing -physdrv%s -array%d -row%d -a0 -NoLog| grep Exit" % (CONFIG.MegaCli64_Path, self.braket, arrayNo, rowNo))
                     run_command("%s -pdrbld -start -physdrv%s -a0 -NoLog" % (CONFIG.MegaCli64_Path, self.braket), 3)
      
                  elif self.statusInitial == "UG" and not '?CACHE' in self.extra:   
                     if CONFIG.confirm:  # usually CONFIG.confirm is in run_command but this is needed to filter out
                        run_command("%s -cfgforeign -clear -a0 -NoLog" % CONFIG.MegaCli64_Path, 3)
                        # %s -pdgetmissing -a0 -NoLog run already above for the purpose of showMessage
                        run_command("%s -pdreplacemissing -physdrv%s -array%d -row%d -a0 -NoLog| grep Exit" % (CONFIG.MegaCli64_Path, self.braket, arrayNo, rowNo))
                        run_command("%s -pdrbld -start -physdrv%s -a0 -NoLog" % (CONFIG.MegaCli64_Path, self.braket), 3)
      
                  run_command("echo > %s/%s.%s.SelfHealingAttempted" % (CONFIG.directory_paths[CONFIG.ibase], self.smart.modelName, self.smart.serialNumber))

         if self.statusInitial == "OFF":
            if "Online" in get_first_word("%s -pdinfo -physdrv%s -a0 -NoLog | grep 'Firmware state'" % (CONFIG.MegaCli64_Path, self.braket)):
               trapMessage += "(Self-healing script has turn on offline drive %s: " % self.statusInitial
            else:
               trapMessage += "(**Needs Review** Turning On Failed %s: " % self.statusInitial
         else:
             if '?CACHE' in self.extra:
                trapMessage += "(**Needs Review** SSD CACHE Issue %s: " % self.statusInitial
             elif "Rebuild" in get_first_word("%s -pdrbld -showprog -physdrv%s -a0 -NoLog | grep Encl" % (CONFIG.MegaCli64_Path, self.braket)):
                trapMessage += "(Self-healing script initiated array rebuild %s: " % self.statusInitial
             else:
                trapMessage += "(**Needs Review** Rebuild Failed %s: " % self.statusInitial

         trapMessage += "%s)" % self.smart_for_trap
         
      return trapMessage
   
      
class Disk_3ware(Disk):
   def __init__(self, appliance, device, nth=-1, slotNo=0, vendor0="AMCC", status="", statusAtUnit="", raidStatus ="", raidname ="", unitname ="", twcli_info_lines=[]): #encId is only for LSI drive5D
      #itwa = int(nth / 12)
      #igrp = int(nth % 12);
      #device = "/dev/twa%d"%itwa
      theIdx = 0
      if "U" in appliance.name: # for 2U and 3U
         lines = twcli_info_lines
         theMax = 0
         cnt = 0
         for line in lines:
            if theMax<int(line.split()[2]):
               theMax = int(line.split()[2])
               theIdx = cnt
            cnt += 1
      device = "/dev/twa%d" % theIdx

      if vendor0 == "LSI":
         device = "/dev/twl0"

      theStatus=status   
      if status=="DEGRADED" and ("REBUILD" in raidStatus or "RBLD" in raidStatus):
         theStatus="REBUILD"
         
      Disk.__init__(self, appliance, device, theStatus)
      self.name = "3ware"

      #self.suffix = ",%02d"%igrp
      #self.hwraid_opt = "-d 3ware,%d" % igrp
      #if vendor0 == "LSI":
      self.suffix = ",%02d"%nth
      self.hwraid_opt = "-d 3ware,%d" % nth
      self.portNo = "p%d"%nth
      self.nameForLogger = "%s" % self.portNo

      self.trapTargAdd =" %s" % self.portNo
      self.trapMessAdd =" [%s]" % self.portNo 

      #VPort Status         Unit Size      Type  Phy Encl-Slot    Model
      #9750 case
      #p8    OK             u0   2.73 TB   SATA  -   /c0/e0/slt0  ST3000DM001-1CH166
      #9650 case
      #p0    OK             u0   2.73 TB   SATA  0   -            ST3000NM0033-9ZM178

      self.slotNo = slotNo  
      self.raidname = raidname
      self.unitname = unitname.strip()

      Disk.FillSMART(self) # filling self.smart as well as self.model

      if self.statusInitial == "TBD":
         if self.smart.realloc >= 0:
            self.statusInitial = "FS"
            self.status = "Failed"
         else:
            self.statusInitial = "F"
            self.status = "Failed"

      elif self.statusInitial == "EE" or self.statusInitial == "DE" or self.statusInitial == "CF":
         if self.smart.realloc >= 0:
            self.statusInitial = self.statusInitial + "S"
      elif self.statusInitial == "DG" and raidStatus == "REBUILDING":
         self.statusInitial = "R"

      # ECC-ERROR with WARNING at unit level is considered as benign  
      if self.statusInitial != "o" and statusAtUnit == "WARNING":
         self.statusInitial = "o"
      # 6AF=   o    u?   /dev/twl0,11    4660 (  194 d)     0     0        8 (042/040> 30: 2785)   ST3000DM001-1CH166                Z1F1YNFD       CC24   3000 GB 0x85

      if self.statusInitial == "o" and statusAtUnit == "NA":  # vport base sorting and it won't get any status if it's untied
         self.statusInitial = "DGS"

      #print "WP:: " + self.statusInitial + " " + statusAtUnit
      Disk.PrepLog(self)
      if (("U" in self.appliance.name or "GE" in CONFIG.assetTag) and unitname != "u0"):
         self.logmid = "E"  # external archive....

      #"F"
      #if raid.type =="3ware":
      #   print "        ==> Please, rescan the drive to find:  tw_cli maint rescan"
      #   print "        ==>         If not found with rescan, then it is most likely dead drive. The more Power_On_Hours, the more likely."   

   def SelfHeal(self, trapMessage, raidLevel, nPendingDisks, nDegradedDisks, progressMessage):  #Disk_3ware

      showMessage(self.name, self.statusInitial, self.raidname, self.unitname, self.portNo)
      #if not CONFIG.selfhealing: return

      if self.statusInitial == "DD": #NOT DEFINED and OBSERVED YET
         trapMessage += "(**Needs Review** %s)" % self.statusInitial 
      elif self.statusInitial == "F" or self.statusInitial == "EE" or self.statusInitial == "DE" or self.statusInitial == "CF":   #Failed and No smart avaiable
         trapMessage += "(**Needs Review** %s)" % self.statusInitial 
      elif self.statusInitial == "R": # This could be DEGRADED
         trapMessage += "(Rebuild Progress %s: %s)" % (progressMessage, self.smart_for_trap)
      else:
         #TODO: at some point you want to avoid any rebuild on rscDrive
         #if self.rscDrive:   
         #else:
         if self.statusInitial == "FS" or self.statusInitial == "EES" or self.statusInitial == "DES" or self.statusInitial == "CFS" or self.statusInitial == "DGS":   
            if CONFIG.selfhealing:
               selfHealable = False
               if raidLevel == "RAID-6":
                  if nPendingDisks<2 and nDegradedDisks<2:
                     if CONFIG.confirm and not os.path.exists("%s/%s.%s.SelfHealingAttempted" % (CONFIG.directory_paths[CONFIG.ibase], self.smart.modelName, self.smart.serialNumber)):
                        selfHealable = True
               elif raidLevel == "RAID-5":
                  if nPendingDisks==0 and nDegradedDisks==1:
                     if CONFIG.confirm and not os.path.exists("%s/%s.%s.SelfHealingAttempted" % (CONFIG.directory_paths[CONFIG.ibase], self.smart.modelName, self.smart.serialNumber)):
                        selfHealable = True

               if selfHealable:         
                  run_command("%s maint remove %s %s" % (CONFIG.tw_cliPath, self.raidname, self.portNo), 3) # sleep 3
                  run_command("%s maint rescan" % CONFIG.tw_cliPath, 3) # sleep 3
                  run_command("%s maint rebuild %s %s %s" % (CONFIG.tw_cliPath, self.raidname, self.unitname, self.portNo), 3) # sleep 3
                  run_command("echo > %s/%s.%s.SelfHealingAttempted" % (CONFIG.directory_paths[CONFIG.ibase], self.smart.modelName, self.smart.serialNumber))

                  
               if selfHealable:
                  if "DEGRADED" == get_nth_word("%s info %s | grep RAID" % (CONFIG.tw_cliPath, self.raidname), 2):
                     trapMessage += "(Self-healing Failed %s: " % self.statusInitial
                  else:   
                     trapMessage += "(Self-healing script initiated array rebuild %s: " % self.statusInitial
               else:  
                     trapMessage += "(**Needs Review** %s: " % self.statusInitial 
            trapMessage += "%s)" % self.smart_for_trap

      return trapMessage

class Disk_OS(Disk):
   def __init__(self, appliance, device, hostNumber, scsi="", flashcache=False): #encId is only for LSI drive

      scsi_model=""
      if scsi: scsi_model=scsi.model
      Disk.__init__(self, appliance, device, "", scsi_model)                              #status is used to pass vendor information in Disk_OS 
         
      firstHostN_OS=CONFIG.firstHostN_AHCI
      deviceShort = device[5:]
      self.hostNumber_suffix,self.hwraid_opt="",""  
      if not "hd" in deviceShort:  # This doesn't hurt and it is needed for some USB3 device
         self.hwraid_opt="-d sat"  #   but it doesn't work with old IDE device in D2000/3000
      self.name = "OS"
      self.hostNumber = hostNumber  # integer
      self.bayNumber = hostNumber-firstHostN_OS
      if self.appliance.flashcache_in_front:
         self.bayNumber -=1

      self.trapTargAdd =" %s"  % self.device
      self.trapMessAdd =" [%s]"% self.device 

      self.nameForLogger = "host%d" % self.hostNumber
      if self.hostNumber_suffix:   # This is due to miniSAS card sharing common host number over multiple devices
         self.nameForLogger += "_%s" % self.hostNumber_suffix
               
      self.flashcache = flashcache
      vendor,self.rotational="",True
      ndisk =appliance.nDisk_OS_and_Cache_for_archive # 602/603 issue host0/2  to cover in between host1 to exclude it
      self.archive = not( (hostNumber>=firstHostN_OS and hostNumber<firstHostN_OS+ndisk) or flashcache)
      if scsi:
         self.hostNumber_suffix =scsi.hostNumber_suffix # This is due to miniSAS card sharing common host number over multiple devices
         vendor =scsi.vendor
         self.rotational=scsi.rotational
         if self.archive:  # Note that degraded one can be considered as archive.. BE CAREFUL
            if scsi.mdraid_OS: self.archive = False
         
      if not len(appliance.oshosts)==1 and not appliance.rebuild_disk_mode and vendor=="ATA":  # New Gen 2014 : No mdadm raid array so every ATA should be considered as Archive drive
         self.archive = True                                 #                Passthrough is connected SCU (SAS Controller Unit)
      if "hd" in deviceShort:
         self.archive = False
      if not self.rotational:
         self.archive = False
      if scsi:
         if scsi.usage=="eSATA": self.archive=True
         
      Disk.FillSMART(self) # filling self.smart as well as self.model
      Disk.PrepLog(self)

      log = ""
      if self.archive:
         log = log + "E"
      else:
         log = log + "="
      self.logmid = log   

      if "SSD" in self.smart.modelName:
         if self.smart.hours > 200000:
            command = "wget ftp://ftp.unitrends.com/utilities/drivedb.h; cp -rf drivedb.h /usr/share/smartmontools/"
            print "SSD SMART DB outdated. Running %s" %command
            print "Please, run it again if you see this message"
            os.system(command)

      self.part   = []
      #self.GetPartition()

   def GetPartition(self):
      # grep -v extend is for legacy disk partition
      for line in get_printout("parted %s unit s print free | grep -A 10 Number | grep s | grep -v Num | grep -v upd | grep -v extend" % str(self.device)):
         if CONFIG.debug: print line
         splited = line.split()
         # 1      34s         104857600s   104857567s                gpt  
         if not splited[0].isdigit():
            continue
         
         smart_num = int(splited[0])
         start = int(splited[1][:-1])
         end   = int(splited[2][:-1])
         size  = int(splited[3][:-1])
         spart = SubPartition(self.device, smart_num, start, end, size) 
         self.part.append(spart)
      else:      
         self.part.append(SubPartition("",0,0,0,0))
         #print str(smart_num) + " " + str(start) + " " + str(end) + " " + str(size)

class DiskError:
   """Holding partition information from smartctl""" 
   def __init__(self, errorId, errorHour, errorName, detailedErrors):
      self.Id = errorId
      self.Hour = errorHour
      self.Type = errorName
      self.RawErrors = detailedErrors
      self.AbbrErrors = []
      self.Abbr = ''
      for err in self.RawErrors:
         AbbrError='N/A'
         if err == 'READ FPDMA QUEUED':
            AbbrError='RFQ'
         elif err == 'WRITE FPDMA QUEUED':
            AbbrError='WFQ'
         elif err == 'FLUSH CACHE':
            AbbrError='FC '
         elif err == 'READ DMA EXT':
            AbbrError='RDE'
         elif err == 'READ NATIVE MAX ADDRESS EXT':
            AbbrError='RAE'
         elif err == 'IDENTIFY DEVICE':
            AbbrError='ID '
         elif err == 'SMART READ DATA':
            AbbrError='SRD'
         elif err == 'SMART RETURN STATUS':
            AbbrError='SRS'
         elif err == 'READ VERIFY SECTOR':
            AbbrError='RVS'
         elif err == 'NOP [Abort queued commands]':
            AbbrError='NOP'
         elif err == 'READ LOG EXT':
            AbbrError='RLE'
         elif err == 'SET FEATURES [Set transfer mode]':
            AbbrError='STM'
         self.AbbrErrors.append(AbbrError)
         
      for abbr in self.AbbrErrors:
          self.Abbr += "%s " % abbr
          
   def __repr__(self):
       return "#%d @%05dH %3s << %s" % (self.Id, self.Hour, self.Type, self.Abbr)
    
#################class SubPartition ##########################
class SubPartition:
   """SubPartition"""

   def __init__(self, device, i, s, e, sz):
      self.id  = i
      self.start = s
      self.end   = e
      self.size  = sz

      self.md = ""
      if os.path.exists("/proc/mdstat"):
         lines = get_printout("grep md /proc/mdstat | grep -v inactive")
         for line in lines: # finding partition associated md array..
            # sometimes no partition defined for md (archive)
            # eg. md6 : active raid0 sdk[3] sdj[2] sdi[1] sdh[0]
            wordsFromline = line.split()
            if len(wordsFromline[4]) < 7: continue                
	    #if wordsFromline[4][3:4].isdigit():	
            if i == int(wordsFromline[4][3:4]):
               self.md = wordsFromline[0]
               break

      self.partition =""   
      lines = get_printout("df | grep '/dev/' ")
      for line in lines:
         wordsFromline = line.split()
         if len(self.md)>0:
            if self.md in wordsFromline[0]:
               self.partition = wordsFromline[5]
         else:
            if "%s"%device + str(i) in wordsFromline[0]:
               self.partition = wordsFromline[5]

   def printout(self):
      return

