from dbmon_Util import buildTrapParams, getTrapParams, sendTrap, SNMPTables, pingHost
from postgres import postgres

def logTrap(trap, message, objectID, setOpen):
	"""
	Inserts a trap into the SNMP_HISTORY table if the database is up for
	each trapserver that is reachable and enabled

	stirng[]{} trap - list of dictionaries that get nessisary data for the trap
	string message - interpolated message that was sent with the trap
	string objectID - objectID that was sent with the trap
	bool setOpen - boolean value that indicates if the trap should be closed

	returns -1 if there were no traps servers enabled or the number of traps
		sent
	"""	

	#Check of the enabled trapservers which are availible
	tc = SNMPTables()
	ntrap_servers = len(trap) - tc.numItems + 1
	if ntrap_servers <= 0:
		return -1
	#print "finish intial traps"
	tservrip = [None]*ntrap_servers
	tservcom = [None]*ntrap_servers
	tenabled = [False]*ntrap_servers
	for i in range(ntrap_servers):
		sec = tc.uniqID[tc.snmp_config]+str(i)
		try:
			tenabled[i] = trap[sec]['enabled'] == 't'
			tservrip[i] = trap[sec]['destination']
			tservcom[i] = trap[sec]['community']
			if tenabled[i]:
				if pingHost(tservrip[i]) < 1:
					tenabled[i] = False
		except:
			pass
		if not tenabled[i]:
			continue
	if not (True in tenabled):
		return -1 

	#Get the information for logging the trap
	oid = 'enterprises.21865.2.100' + trap[tc.trap_types]['trap_type_id']
	severity = trap[tc.notifications]['severity']
	traptype = trap[tc.trap_types]['trap_type']

	#loop over the trap servers that were enabled and availible and log them
	traps_sent = 0
	for i in range(ntrap_servers):
		if tenabled[i]:
			community = tservrip[i]
			destination = tservrip[i]
			pgquery = ("insert into bp.snmp_history (oid," +
				"destination,community,traptype,object,description,severity,status)" +
				"values ('%s','%s','%s','%s','%s','%s',%s, %d)" )% (
				oid,destination,community,traptype,objectID,message,severity, setOpen)
			try:
				db = postgres()
				db.query(pgquery)
			except:
				pass
			else:
				traps_sent += 1

	return traps_sent

def pySendNotification(notifi_id = -1, value = (), severity=3, primary="", state='f', configFile='/usr/bp/schema/.dbmon_tableDump'):
	"""
	Interface for sending both traps from the database and custom traps
	
	Keyword Arguments
	int notifi_id - notification id number; use -1 for custom trap
	Tuple value   - tuple for interpolation
	int severity  - severity for a custom trap
	string primary- String to be sent as the message for the custom trap
	char state	- 't' if the custom trap is to be open; 'f' if it is not
	string configFile - path to the cached database

	Throws:
	ConfigParser.NoSectionError if the requested notifiation id does not exist
	"""
	if (notifi_id == -1):
		trap = buildTrapParams(-1, '20', '20', severity, primary, "", "", "onDemand", "onDemand", state,'f','f','f','f','f')
	else:
		trap = getTrapParams(configFile, notification_id = notifi_id)
	#print trap
	message  = trap[0]['primary_text'] % value
	objectID = '100.0.' + str(trap[2]['trap_type_id'])
	setOpen  = trap[0]['state'] == 'f'
	return_code = sendTrap(message, objectID, trap, setOpen)
	if return_code > 150 :
		logTrap(trap,message,objectID,setOpen)
	return return_code

