import sys
if not '/usr/bp/bin' in sys.path:
	sys.path.append('/usr/bp/bin')
from dbmon_Util import runCommand, DBUtil, readSamples
from dbmon_UpdateConfig import runQueryValidateData

class postgres:
	"""
	Class for interfacing with the database for live queries
	"""
	user = ""
	db   = ""
	tmp  = ""

	def __init__(self, user='postgres', db='bpdb', tmp='testing.csv'):
		"""
		initizalize the database interface object
		
		Keyword Arguments:
		string user - username to access the postgres db
		string db - database to access
		string tmp - tempfile to use to parse the results
		"""
		self.user = user
		self.db = db
		self.tmp = tmp
		if self.user == "" or self.db == "" or self.tmp == "":
			raise TypeError

	def select(self, query):
		"""
		runs a query of the form 'select *' on the postgres database

		Keyword Arguments
		string query - the items to select from the database; must be of the form:
			'* from bp.alerts' - get all of the alerts 
			'* from bp.alerts where alert_id = 56' get all of the alert with id 56

		Returns:
		tuple (status, data)
			int status - error code that contains 1 if there were no errors or 0 if
				there was an exception
			string[]{} data - a list of dicionary entries where each colum names
				are keys is the key.
			Data will be None if there was an error or no results from the query

		Throws:
		DatabaseNotUp if the database cannot be reached
		"""
		db_Status = DBUtil()
		if not (db_Status.isDBUp() == 1 and db_Status.pingDB() ==1):
			raise DatabaseIsDown
		psqlquery = 'psql %s %s -XA -P footer=off -c "select %s"' % (self.db, self.user, query)
		countquery = psqlquery.replace("*", "count(*)",1)
		countquery = countquery.replace("-XA", "-XAt",1)
		psqlquery += '> %s 2> /dev/null' % self.tmp
		return runQueryValidateData(psqlquery, countquery, self.tmp)
	
	def query(self, query):
		"""
		runs a command in the postgres database

		Keyword Arguments,
		string query - query to run in the postgres database

		Returns:
		String containing the output of the command

		Throws:
		DatabaseIsDown is the database cannot be reached
		"""
		
		db_Status = DBUtil()
		if not (db_Status.isDBUp() == 1 and db_Status.pingDB() ==1):
			raise DatabaseIsDown
		psqlquery = 'psql %s %s -XA -P footer=off -c "%s"' % (self.db, self.user, query)
		out, err, pipe, res = runCommand(psqlquery)
		return out

class DatabaseError(Exception):
	"""
	BaseError for all database Errors
	"""

class DatabaseIsDown(DatabaseError):
	"""
	Database is either not running or is not pingable
	"""


