import os,sys   
import ConfigParser
abs_path = os.path.abspath(sys.argv[0])
basename = os.path.basename(sys.argv[0])
dirname = abs_path.replace(basename, "")
if not "%s/diskmon/lib" % dirname in sys.path:
   sys.path.insert(0,"%s/diskmon/lib" % dirname) # always with regard to parent script
if not "/usr/bp/lib/monitoring" in sys.path:
   sys.path.append("/usr/bp/lib/monitoring")
from disk      import *   # include config << bpdb as well 
from support_message      import *   # include config << bpdb as well 

#raidInfo code:
#Ex=0: healthy
#Ex=1: rebuild in progress
#Ex=2: verify operation in progress
#Ex=3: verify  is initiated
#Ex=4: degraded
#Ex=5: replace the drive
# (routine evalution of hard disk subsystems in progress)


def get_state_index(state):
   states = ['OPTIMAL', 'PARTIALLY DEGRADED', 'DEGRADED', 'OFFLINE']
   for i,s in enumerate(states):
      if s == state:
         return i

def find_word_wrt_key(key, link, splitter="", offset=0):
   if len(splitter)==0:
      words = link.split()
   else: 
      words = link.split(splitter)
   idx, theIdx =0, 0
   for word in words:
      if key in word:
         theIdx = idx
      idx = idx + 1

   return words[theIdx+offset]

def doRaidInfo(pendingDisks, problemDisks):

   allowedModels=['710','711','712','713','812','813','713S','714S','814S']

   #if len(self.pendingDisks) == len(self.problemDisks) or CONFIG.testOn == "raidInfo":  # raidInfo repair
   if CONFIG.modelNo not in allowedModels: return False
         
   if not CONFIG.selfhealing: return False
   if not CONFIG.confirm:     return False # FILTER IS NEEDED TO AVOID OVER AND OVER TRY
   if not CONFIG.Enable_raidInfo: return False  # default is True
   
   if CONFIG.testOn == "raidInfo":
      if CONFIG.verbose == "raidInfo":
         print CONFIG.modelNo, allowedModels
      return True
   
   if not (len(pendingDisks)): return False
   if not (len(problemDisks)): return False
   if len(pendingDisks) != len(problemDisks): return False
   
   return True

#################class RAID#########################
class RAID:
   def __init__(self, name, ndisk, status, depth=-1):
      self.name  = name
      self.ndiskPerSpan = ndisk

      self.depth =1
      if depth >1: self.depth=depth
      
      self.ndisk = ndisk*depth
      self.status = status        #Optimal, Partically Degraded, Degraded
      self.nmiss = 0
      self.disks =[]
      self.degradedDisks = []
      self.reallocDisks =[]
      self.serDisks =[]
      self.pendingDisks =[]
      self.problemDisks = [] # either reallocDisk, serDisk, pendingDisk
      self.needDisks = []    # either pendingDisk, degradedDisk
      self.deadDisks = []

      self.message = name + " ++++ %s" % status

      if status!="Optimal" and status !="OK":
          status = red(status)

      self.printMessage = name + " ++++ %s" % status
      self.progressMessage =""

      self.diskConfig = ConfigParser.ConfigParser()
      if os.path.exists(CONFIG.configFile):
         self.diskConfig.read(CONFIG.configFile)
      

######################      
class RAID_LSI(RAID):
   def __init__(self, vdname, device, ndisk, size, status, raidLevel, cachePolicy, depth=-1, isVDCached=False, bgiProgress=""):
      self.VD_Id = int(vdname[-1:])
      self.device = device
      self.isVDCached=isVDCached
      self.bgiProgress = bgiProgress
      
      segs=size.split()
      theSize=float(segs[0])
      TB=theSize
      if segs[1]=="GB":         TB /= 1024.
      if segs[1]=="TB":    theSize *= 1024
         
      self.size = theSize   # GB
      self.TB=int(TB+0.5)
      
      self.raidLevel= (raidLevel.split(",")[0])[-1:]
      self.writemode= cachePolicy.split(",")[0].strip()
      if depth==2: self.raidLevel += "0"

      name = vdname
      RAID.__init__(self, name, ndisk, status, depth)  # self.ndisk = ndisk*depth

      self.healthy = not "Degrade" in self.status and not "Offline" in self.status
      words = name.split()
      first = words[0]
      second = words[1]
      third = words[2]

      effNDisk=self.ndisk
      if self.raidLevel == "1":         effNDisk /= 2
      if self.raidLevel == "6":         effNDisk -= 2
      if self.raidLevel == "60":        effNDisk -= 4

      self.diskSize = self.size/effNDisk

      self.shortName = first[0] + second[0] + third #Virtual Drive: 0
      self.progressMessage = ""
      if len(self.bgiProgress):
         self.progressMessage = "(Background Initialization: %s)" % self.bgiProgress
      if CONFIG.testOn == "LSIBGI":
         self.progressMessage = "(Background Initialization: 10%)"
         
         
   def __repr__(self):
      if "Cache" in self.name:
         return "%s (%s) ++++ %s" % (self.name, self.writemode, self.status)
      else:
         log = "RAID %s (RAID-%s: %s) ++++ %s" % (self.name, self.raidLevel, self.writemode, self.status)
         if len(self.progressMessage):
            log += " " + self.progressMessage
         return log   
         
   def alert(self): #LSI
      if "raid" in CONFIG.verbose:
         print "raid.py:: ", self.ndisk, self.nmiss, self.size, len(self.degradedDisks), len(self.problemDisks)

      if self.healthy:  # RAID is Okay
         #WP Delete All the Files...
         rets = get_printout("ls /var/log/monitoring_disks/*SelfHealingAttempted 2>&1| grep -v 'No such'")
         if len(rets)>0:
            os.system("mv %s/*SelfHealingAttempted %s/" % (CONFIG.directory_paths[CONFIG.ibase], CONFIG.directory_paths[CONFIG.iretired]))

         if len(self.problemDisks)==0 and not CONFIG.testOn == "raidInfo":
             CloseAnyActiveAlerts(self.shortName) # shortName is usually VD name
         else:
            retIdx =9
            if doRaidInfo(self.pendingDisks, self.problemDisks):
               theCommand="%s diskmon" % CONFIG.raidInfo_Path
               if CONFIG.testOn == "raidInfo":
                  theCommand="%s test" % CONFIG.raidInfo_Path
               lines=get_printout(theCommand)
               if CONFIG.testOn == "raidInfo":
                  print theCommand,lines
               for line in lines:
                  if len(line) !=1: continue
                  retIdx =int(line)
                  break
                  
            if retIdx >3:   
               trapTarget  = self.shortName + " SMT"
               trapMessage = self.name + " SMART concerns due to disk(s):"
               for disk in self.problemDisks:
                  trapTarget  += disk.trapTargAdd 
                  trapMessage += disk.trapMessAdd + "(**Needs Review**: %s)" % disk.smart_for_trap
               if CONFIG.testOn == "raidInfo":
                  trapTarget  += "Test"
                  trapMessage += "(**Needs Review**)"
                  
               trapMessage += " >>%d" % retIdx   
               if CONFIG.selfhealing: # delayedAlert handles... and CONFIG.dbON:
                  notification_id = get_notification_id(trapMessage)                  
                  HandleAlert(self.shortName, notification_id, trapTarget, trapMessage) # shortName is usually VD name
               
      else: #if "Degrade" in self.status or "DEGRADE" in self.status:
         trapTarget  = self.shortName
         trapMessage = self.message + " due to disk(s):"

         if "raid" in CONFIG.verbose: print "raid.py:: ", len(self.degradedDisks), len(self.problemDisks)

         if len(self.degradedDisks):  # MEANING: DEGRADE and Drive is recognized
            for disk in self.degradedDisks:
               trapTarget  += disk.trapTargAdd + "%s" % disk.statusInitial 
               trapMessage += disk.trapMessAdd
               trapMessage = disk.SelfHeal(trapMessage) ##############
         else:                        # MEANING: DEGRADE and Drive is dead
            lines=[]
            if self.nmiss:
               lines =get_printout("ls -t /var/log/monitoring_disks/old/*dat 2>&1 | grep -v cannot")
            for i in range(len(lines)):
               if self.nmiss == i: break
               line=lines[i]
               words=line.split(".")
               segs = words[0].split("/")
               theModel = segs[len(segs)-1]
               theSN = words[1]
               trapTarget += " B99DD"
               trapMessage += " [Bay99](**Needs Review** DD)"
               
         if len(self.problemDisks)!=0:
            trapTarget  += " SMT"
            trapMessage += " ++++ SMART concerns due to disk(s):"
            for disk in self.problemDisks:
               trapTarget  += disk.trapTargAdd
               trapMessage += disk.trapMessAdd + "(**Needs Review**: %s)" % disk.smart_for_trap

         if CONFIG.selfhealing: # delayedAlert handles... and CONFIG.dbON:
            notification_id = get_notification_id(trapMessage)
            HandleAlert(self.shortName, notification_id, trapTarget, trapMessage) # shortName is usually VD name

      ###
      if len(self.problemDisks)!=0:
         if len(self.pendingDisks)==len(self.problemDisks):
            showMessage("LSI", "SMTP")
         else:
            showMessage("LSI", "SMT")

      return self.nmiss or len(self.degradedDisks)

###################
class RAID_3ware(RAID):
   def __init__(self, name, ndisk, status, depth=-1, nNotOpt =-1, raidlevel="", subunits=[]):
      RAID.__init__(self, name, ndisk, status, depth)
      self.nmiss = nNotOpt
      self.shortName = name
      self.subunits = subunits
      self.raidLevel = raidlevel
      self.tw_cli_output = get_printout("%s info %s" % (CONFIG.tw_cliPath, name))
      self.tw_cli_unitstatus=""
      for line in self.tw_cli_output:
         if not "RAID" in line: continue
         self.tw_cli_unitstatus =line
         
      self.writemode  = self.tw_cli_unitstatus.split()[7]
      self.autoverify = self.tw_cli_unitstatus.split()[8]
      self.printMessage =""
      
         
      nOKDrives =0
      nDGDrives =0
      for ue in subunits: # this is for port list
         if ue.status == "NOT-PRESENT":
            continue 
         if ue.status == "OK":
            nOKDrives += 1
         else:  # if ue.status == "DEGRADED": there is "-"
            nDGDrives += 1

      self.nOKDrives = nOKDrives
      self.nDGDrives = nDGDrives

      if status == "REBUILDING" or status == "REBUILD-VERIFY" or "RBLD" in status or "REBUILD-INIT" in status:  #DEGRADED-RBLD
         self.progressMessage = self.tw_cli_unitstatus.split()[3]
         self.printMessage = self.message + " " + self.progressMessage
      elif "INIT" in status or status == "VERIFYING":  # INITIALIZING or DEGRADED-INIT
         self.progressMessage = self.tw_cli_unitstatus.split()[4]
         self.printMessage = self.message + " " + self.progressMessage

      self.healthy =not "DEGRADED" in self.status

         
   def __repr__(self):
      return "RAID %s (%s: %s,%s) ++++ %s %s" % (self.name, self.raidLevel, self.writemode, self.autoverify, self.status, self.printMessage)


   def alert(self): # 3ware
      if self.healthy:
         #WP Delete All the Files...
         rets = get_printout("ls /var/log/monitoring_disks/*SelfHealingAttempted 2>&1| grep -v 'No such'")
         if len(rets)>0:
            os.system("mv %s/*SelfHealingAttempted %s/" % (CONFIG.directory_paths[CONFIG.ibase], CONFIG.directory_paths[CONFIG.iretired]))

         if len(self.problemDisks)==0 and not CONFIG.testOn == "raidInfo":
            CloseAnyActiveAlerts(self.shortName)
         else:
            retIdx =9
            if doRaidInfo(self.pendingDisks, self.problemDisks):
               theCommand="%s diskmon" % CONFIG.raidInfo_Path
               if CONFIG.testOn == "raidInfo":
                  theCommand="%s test" % CONFIG.raidInfo_Path
               lines=get_printout(theCommand)
               if CONFIG.testOn == "raidInfo":
                  print theCommand,lines
               for line in lines:
                  if len(line) !=1: continue
                  retIdx =int(line)
                  break
               
            if retIdx >3:   
               trapTarget = self.shortName + " SMT"
               trapMessage = self.name + " SMART concerns due to disk(s):"
               for disk in self.problemDisks:
                  trapTarget  += disk.trapTargAdd
                  trapMessage += disk.trapMessAdd + "(**Needs Review**: %s)" % disk.smart_for_trap
               if CONFIG.testOn == "raidInfo":
                  trapTarget  += "Test"
                  trapMessage += "(**Needs Review**)"

               trapMessage += " >>%d" % retIdx   
               if CONFIG.selfhealing: #delayedAlert handles... and CONFIG.dbON:
                  notification_id = get_notification_id(trapMessage)
                  HandleAlert(self.shortName, notification_id, trapTarget, trapMessage) # shortName is usually controller name
                  
      else: #if "Degrade" in self.status or "DEGRADE" in self.status:
         trapTarget  = self.shortName
         trapMessage = self.message + " due to disk(s):"
                                    
         nPendingDisks = len(self.pendingDisks)
         nDegradedDisks = len(self.degradedDisks)
         
         for disk in self.degradedDisks:
            trapTarget  += disk.trapTargAdd + "%s" % disk.statusInitial 
            trapMessage += disk.trapMessAdd
            trapMessage = disk.SelfHeal(trapMessage, self.raidLevel, nPendingDisks, nDegradedDisks, self.progressMessage) ##############

         if len(self.problemDisks)!=0:
            trapTarget  += " SMT"
            trapMessage += " ++++ SMART concerns due to disk(s):"
            for disk in self.problemDisks:
               trapTarget  += disk.trapTargAdd
               trapMessage += disk.trapMessAdd + "(**Needs Review**: %s)" % disk.smart_for_trap
                           
         if CONFIG.selfhealing: # delayedAlert handles.... and CONFIG.dbON
            notification_id = get_notification_id(trapMessage)
            HandleAlert(self.shortName, notification_id, trapTarget, trapMessage) # shortName is usually controller name

      if len(self.problemDisks)!=0:
          showMessage("3ware", "SMT")
          
      return len(self.degradedDisks)

def get_mdraids():
   devices = [] # this is from /proc/mdstat
   mdraids = []
   status = "OK"
   statcnt =0

   command="grep -v Personalities /proc/mdstat"
   if CONFIG.testOn == "SWRAID":
      command="cat /root/benchmark/swraid.out"
   for line in get_aggregated_lines(command):
      if not "U" in line: continue
      estimate, substatusBit, progress, substatus = "", "", "Done", "OK" 
      subdevices, subdeviceStatus =[], []
      words = line.split()
      for word in words:
         if "U" in word:
            substatusBit = word[1:len(word)-1]
            if "recovery" in line:   # re-add / add: you will see recovery  
               substatus = "recovery"
            elif "resync" in line:   # mdadm v3.2.6  recovery status considered as U and called resync
               substatus = "resync"  # resync=DELAYED
         if "_" in word:
            if "recovery" in line:
               substatus = "recovery"
            elif "resync" in line:
               substatus = "resync"  # resync=DELAYED
            elif "(F)" in line:
               substatus = "failed"
               progress = "NotInitiated"
               statcnt += 1
            else:   
               substatus = "Degraded"
               progress = "NotInitiated"
               statcnt += 1
         elif "md" in word:
            raidName = word
         elif "%" in word:
            progress = word
         elif "DELAYED" in word:
            progress = "DELAYED"
         elif "finish=" in word:
            estimate = word
         elif "sd" in word:
            device = ""
            if "(F)" in word:
               subdevices.append(word[0:len(word)-6])
               subdeviceStatus.append("F")
               device = word[0:len(word)-7]
            else:
               subdevices.append(word[0:len(word)-3])
               subdeviceStatus.append("O")
               device = word[0:len(word)-4]
            if not device in devices:
               devices.append(device)
      mdraids.append(MdRaid(raidName, substatus, substatusBit, progress, subdevices, subdeviceStatus, estimate))

   if statcnt > 0:
       status = "Degraded"
   return (status, devices, mdraids)    


########
class RAID_OS(RAID):
   def __init__(self, appliance): #, config):
      #self.config = config
      ###
      (status, devices, mdraids) =get_mdraids()   
      mdraids.sort(key=lambda MdRaid: MdRaid.name)

      self.N_DegMdRaid, self.N_RebuildMd = 0,0
      for mdraid in mdraids:
         if "Degraded" == mdraid.status: self.N_DegMdRaid+=1
         if "resync"   == mdraid.status: self.N_RebuildMd+=1
         if "recovery" == mdraid.status: self.N_RebuildMd+=1
      
      depth = 1
      if "7" in appliance.name:
         depth = 2

      ndisk = appliance.nDisk_OS_and_Cache
      if "0" == get_first_word("grep -v Personalities /proc/mdstat |grep -v unused| /usr/bin/wc -l") and appliance.controller == "":  # single drive appliance such as 201  
         ndisk = 1
         for line in get_printout("mount | grep dev | grep sd | grep boot"):
            devices.append(line.split()[0][5:8])
            
      if CONFIG.verbose=="RAID_OS":
         print "WP@raid.py: ndisk appliance.mdlevel appliance.name ", ndisk, appliance.mdlevel, appliance.name
         print "WP@raid.py: CentOS_version ", CONFIG.CentOS_version
         for raid in mdraids:
            print "mdraids ;;;", raid
         
      firstHostN_OS=CONFIG.firstHostN_AHCI
      total_Ndisk =len(appliance.oshosts)
      if appliance.flashcache: total_Ndisk += 1
      self.ssdHosts=[]
      liveHosts, liveDevices, md_device_candidates =[],[],[]

      for scsi in CONFIG.scsi_devices:
         if not ("ATA" in scsi.vendor or "WDC WD" in scsi.vendor or ("ST" in scsi.vendor) or "JMicron" in scsi.vendor): continue
         # REJECT SSD   
         if "0" == get_first_word("cat /sys/block/%s/queue/rotational" % scsi.deviceShort):
            self.ssdHosts.append("host%d" % scsi.hostNumber)
            continue  
         if scsi.hostNumber >=0:            liveHosts.append(scsi.hostNumber)
         if len(scsi.deviceShort):          liveDevices.append(scsi.deviceShort)
         if scsi.hostNumber >= firstHostN_OS and scsi.hostNumber <firstHostN_OS+total_Ndisk:
            md_device_candidates.append(scsi.deviceShort)
            
      self.missingHosts = []
      if firstHostN_OS >=0:
          for idHost in range(firstHostN_OS,firstHostN_OS+total_Ndisk):
             if idHost in liveHosts: continue
             theHost ="host%d"%idHost
             if theHost in self.ssdHosts: continue
             self.missingHosts.append(theHost)

      # bug 20324 case: host numbers are host6/7
      self.mdraids = mdraids
      self.disks, self.shortName = [], "OS"
      self.md_devices=devices # this is from mdstat
      self.md_device_candidates=md_device_candidates
      self.devices = liveDevices  # live devices including flashcache in new gen


      RAID.__init__(self, "OS", ndisk, status, depth)
      self.ndisk = ndisk
      self.nmiss = ndisk - len(self.devices) -len(self.ssdHosts)  # devices are from /proc/mdstat so that nmiss and len(self.missingHosts) become inconsistent
      self.message = "OS RAID ++++"
      self.healthy = not "Degraded" in self.status
      
      if CONFIG.verbose=="RAID_OS":
         print "WP@raid.py: ndisk ", ndisk
         print "WP@raid.py: nmiss", self.nmiss
         print "WP@raid.py: md_devices ", devices 
         print "WP@raid.py: md_device_candidates ", md_device_candidates
         print "WP@raid.py: liveHosts ", liveHosts 
         print "WP@raid.py: liveDevices (self.devices)", liveDevices 
         print "WP@raid.py: missingHosts ", self.missingHosts
         print "WP@raid.py: ssdHosts ", self.ssdHosts

   def __repr__(self):
      log =""
      for i  in range(len(self.mdraids)):
         mdraid=self.mdraids[i]
         log += "%s" % mdraid
         if i+1<len(self.mdraids):
            log +="\n"
      return log   
         
   def alert(self): ### RAID_OS
      import commands
      if self.healthy:
         if len(self.problemDisks)==0 and not CONFIG.testOn == "raidInfo":
            if not self.N_RebuildMd:
               CloseAnyActiveAlerts(self.shortName)
               #WP Delete All the Files...
               rets = get_printout("ls /var/log/monitoring_disks/*SelfHealingAttempted_md* 2>&1| grep -v 'No such'")
               if len(rets)>0:
                  os.system("mv %s/*SelfHealingAttempted_md* %s/" % (CONFIG.directory_paths[CONFIG.ibase], CONFIG.directory_paths[CONFIG.iretired]))
            else:  # when rebuild is running
               trapTarget  = self.shortName 
               trapMessage = self.message + " (Rebuild Progress)"
               notification_id = 138
               HandleAlert(self.shortName, notification_id, trapTarget, trapMessage) # shortName is usually VD name

         else:
            retIdx =9
            if doRaidInfo(self.pendingDisks, self.problemDisks):
               theCommand="%s diskmon" % CONFIG.raidInfo_Path
               if CONFIG.testOn == "raidInfo":
                  theCommand="%s test" % CONFIG.raidInfo_Path
               lines=get_printout(theCommand)
               if CONFIG.testOn == "raidInfo":
                  print theCommand,lines
               for line in lines:
                  if len(line) !=1: continue
                  retIdx =int(line)
                  break
                  
            if retIdx >3:   
               trapTarget = self.shortName + " SMT"
               trapMessage = self.name + " SMART concerns due to disk(s):"
               for disk in self.problemDisks:
                  trapTarget  += disk.trapTargAdd
                  trapMessage += disk.trapMessAdd + "(**Needs Review**: %s)" % disk.smart_for_trap
               if CONFIG.testOn == "raidInfo":
                  trapTarget  += "Test"
                  trapMessage += "(**Needs Review**)"

               trapMessage += " >>%d" % retIdx   
               if CONFIG.selfhealing: #delayedAlert handles... and CONFIG.dbON:
                  notification_id = get_notification_id(trapMessage)
                  HandleAlert(self.shortName, notification_id, trapTarget, trapMessage) # shortName is usually OS name
               
      else: #if "Degrade" in self.status or "DEGRADE" in self.status:
         trapTarget  = self.shortName
         trapMessage = self.message + " Degraded"
         
         if self.nmiss > 0: # in case there is missing drive!!!!
            trapMessage += " due to offline disk"
            for i in range(len(self.missingHosts)):
               host = self.missingHosts[i]
               trapTarget += " " + host
               theModel=self.diskConfig.get(host, "Model")
               theSN=self.diskConfig.get(host, 'Serial')
               if "NotAvailable" in theModel:
                  ls_command = "ls -t %s/*.dat" % (CONFIG.directory_paths[2])
                  #print i, len(self.missingHosts), self.nmiss, ls_command
                  diskFile = get_printout(ls_command)[i]
                  words=diskFile.split("/")
                  segs = words[len(words)-1].split(".")
                  theModel=segs[0]
                  theSN=segs[1]
               trapMessage += " " + host + "(Model:%s" % theModel + " Serial:%s)" % theSN
              
            trapMessage += " (**Needs Review** DD)" 
            if not CONFIG.selfhealing:
               log = "[ALERT] Degraded due to the offline disk"
               for i in range(len(self.missingHosts)):
                  host = self.missingHosts[i]
                  theModel=self.diskConfig.get(host, "Model")
                  theSN=self.diskConfig.get(host, 'Serial')
                  if "NotAvailable" in theModel:
                     ls_command = "ls -t %s/*.dat" % (CONFIG.directory_paths[2])
                     diskFile = get_printout(ls_command)[i]
                     words=diskFile.split("/")
                     segs = words[len(words)-1].split(".")
                     theModel=segs[0]
                     theSN=segs[1]
                  log = log + " " + host + " (Model: %s" % theModel + " Serial: %s)" % theSN
               print log    
               print "        ==> Please, initiate warranty request ONLY IF NEEDED"
               print "        ==> Useful commands : ls -l /sys/block/sd*/device"
         else:  #   # doable selfhealing candidate
            diskProblem = len(self.reallocDisks) or len(self.serDisks)  # see if disk drive has an issue
            nDegraded, nAttempted = 0, 0
            for mdraid in self.mdraids:
               if mdraid.status == "Degraded":   nDegraded += 1
               if diskProblem: continue  # with disk problem (realloc or ser) we don't want to proceed self-healing
               if self.ndisk != 2: continue  # only for RAID 1 with 2 drives 
               if not CONFIG.selfhealing or not CONFIG.confirm: continue # FILTER IS NEEDED TO AVOID OVER AND OVER TRY
               ##################
               # SelfHealing part   
               ##################
               if os.path.exists("%s/SelfHealingAttempted_%s" % (CONFIG.directory_paths[CONFIG.ibase], mdraid.name)):
                   continue
               Attempted = False
               if mdraid.status == "Degraded":
                   if len(mdraid.deviceStatus) == 2 : continue
                   #print self.devices, mdraid.devices, mdraid.deviceStatus
                   #['sdb', 'sdc'] ['sdb2'] ['O']
                   deviceNInRaid = mdraid.devices[0] # sdb2
                   deviceInRaid  = deviceNInRaid[:-1] # sdb
                   subPartId = deviceNInRaid.replace(deviceInRaid,"") #2

                   deviceToAdd = ""
                   for dev in self.devices: # can't move to mdraid class due to this self.devices!!! :(
                       if dev == deviceInRaid: continue
                       deviceToAdd = dev

                   if not len(deviceToAdd): continue

                   command = "/sbin/mdadm --add /dev/%s /dev/%s%s"  % (mdraid.name, deviceToAdd, subPartId)
                   run_command(command)
                   Attempted = True
                   nAttempted += 1
               elif mdraid.status == "failed":
                   if len(mdraid.deviceStatus) == 1 : continue
                   #print self.devices, mdraid.devices, mdraid.deviceStatus
                   #['sdb', 'sdc'] ['sdb3', 'sdc3'] ['O', 'F']
                   deviceToTreat = ""
                   for i in range(len(mdraid.devices)):
                       if mdraid.deviceStatus[i] == "F":
                           deviceToTreat = mdraid.devices[i]
                   if not len(deviceToTreat): continue
                   command = "/sbin/mdadm --remove /dev/%s /dev/%s"  % (mdraid.name, deviceToTreat)
                   run_command(command)
                   command = "/sbin/mdadm --add /dev/%s /dev/%s"  % (mdraid.name, deviceToTreat)
                   run_command(command)

                   Attempted = True
                   nAttempted += 1
               if Attempted:     
                   run_command("echo > %s/SelfHealingAttempted_%s" % (CONFIG.directory_paths[CONFIG.ibase], mdraid.name))


            messageString =""
            if nDegraded and nDegraded == nAttempted:
               messageString = "Rebuild in progress"
               if nAttempted:
                  messageString = "Self-healing script initiated array rebuild +%d" % nAttempted
               showMessage("OS", "RBLD")
            else:
               messageString ="**Needs Review**"
               if CONFIG.verbose:
                  print "WP trapmessage : ", self.md_devices, self.md_device_candidates

            ### If no degraded disk, then this will catch the device      
            degraded_device =list( set(self.md_device_candidates) - set(self.md_devices) )
            if len(degraded_device):
               for device in degraded_device:
                  for disk in self.disks:
                     if not device in disk.device: continue
                     trapMessage += disk.trapMessAdd + "(%s: %s)" % (messageString, disk.smart_for_trap)
               showMessage("OS", "FS")
            elif self.N_DegMdRaid:
               trapMessage += " mdraid(s) "
               for mdraid in self.mdraids:
                  if mdraid.status=="Degraded":
                     trapMessage += " %s" % mdraid.name
               trapMessage += " Degraded (**Needs Review** PD)"
                     
         if len(self.problemDisks)!=0:
            trapTarget  += " SMT"
            trapMessage += " ++++ SMART concerns due to disk(s):"
            for disk in self.problemDisks:
               trapTarget  += disk.trapTargAdd
               trapMessage += disk.trapMessAdd + "(**Needs Review**: %s)" % disk.smart_for_trap
               
         if CONFIG.testOn=="TRAP" or CONFIG.testOn=="SWRAID":
            print trapMessage
               
         if CONFIG.selfhealing: # dbOFF handle belong to delayedAlert  hanand CONFIG.dbON:
            notification_id = get_notification_id(trapMessage)
            HandleAlert(self.shortName, notification_id, trapTarget, trapMessage) # shortName is usually VD name
      ###      
      if len(self.problemDisks)!=0:
          showMessage("OS", "SMT")
          
      return (self.status == "Degraded")    

def get_notification_id(trapMessage):
   notification_id = 0
   if "Needs Review" in trapMessage: # severity becomes 1
      notification_id = 137
   else:   #severity becomes 3
      notification_id = 138
   return notification_id

class MdRaid: 
   """MdRaid which will be subset of RAID_OS""" 
   def __init__(self, name, status, bit, progress, devices, deviceStatus, estimate): #encId is only for LSI drive
      self.name = name
      self.status = status
      self.statusBit = bit
      self.progress = progress
      self.devices = devices
      self.deviceStatus = deviceStatus
      self.estimate = ""
      self.stripe_cache_size =""

      if os.path.exists("/sys/block/%s/md/stripe_cache_size" % self.name):
         self.stripe_cache_size=get_first_word("cat /sys/block/%s/md/stripe_cache_size" %self.name)

      self.getra=get_first_word("/sbin/blockdev --getra /dev/%s" % self.name)
      
      if len(estimate)>9:
 	 est = int(float(estimate[7:-3]))
         import datetime
         now = datetime.datetime.now()
         then = now + datetime.timedelta(minutes = est)
         self.estimate = "by %02d-%02d %02d:%02d" %(then.month, then.day, then.hour, then.minute) 

      hosts=[]
      subwords =[]

      for i in range(len(devices)):
         device = devices[i] 
         host = get_hostnumber(device[0:-1])
         hosts.append(host)
         if "F" in self.deviceStatus[i]:
             subwords.append("%s(F)(%s)" % (device,host))
         else:    
             subwords.append("%s(%s)" % (device,host))

      self.hosts = hosts
      self.subwords = subwords 


   def __repr__(self):
      log = "RAID /dev/%s" %self.name + " ++++ %8s" % self.status
      if "re" in self.status:   
         log += "%9s" % (self.progress)
      else:   
         log += "         "

      log += "    current disks:"
      temp = self.subwords
      temp.sort()
      for subword in temp:
         log = log + " " + subword 
      log += " %s" % (self.estimate)

      log += " ra:%s " %(self.getra)
      if len(self.stripe_cache_size):  
         log += " stripe:%s " %(self.stripe_cache_size)

      return log

