import os,sys
abs_path = os.path.abspath(sys.argv[0])
basename = os.path.basename(sys.argv[0])
dirname = abs_path.replace(basename, "")
if not "%s/diskmon/lib" % dirname in sys.path:
   sys.path.insert(0,"%s/diskmon/lib" % dirname) # always with regard to parent script
if not "/usr/bp/lib/monitoring" in sys.path:
   sys.path.append("/usr/bp/lib/monitoring")

from util      import *   # include config << bpdb as well 

################class Trap ############################
class Trap:
    def __init__(self, category_id, identifier, message, severity, status, notification_id=999):

 #notification_id | category_id |       short_description       | primary_text | severity | state | action | alert | trap | email | psa | log
 #----------------+-------------+-------------------------------+--------------+----------+-------+--------+-------+------+-------+-----+-----
 #  137 |          15 | Disk health state has changed | %s           |                  1 | f     |        | t     | t    | f     | t   | t
 #  138 |          15 | Disk health state has changed | %s           |                  3 | f     |        | t     | t    | f     | f   | t
 #  139 |          15 | Disk health trap is collected | %s           |                  3 | t     |        | f     | t    | f     | f   | f
 #  140 |           0 | Internal notice | Resolve/Close alerts for Disk Health source|  0 | t     |        | t     | f    | f     | f   | f
 #  155 |          16 | IPMI MIN Event                | "%s"         |                  3 | f     |        | f     | t    | t     | f   | f
 #  156 |          16 | IPMI MAJ Event                | "%s"         |                  2 | f     |        | f     | t    | t     | t   | f
 #  157 |          16 | IPMI CRT Event                | "%s"         |                  1 | f     |        | f     | t    | t     | t   | f
 #  158 |           0 | Internal notice | Resolve/Clear alerts for IPMI events  |       0 | t     |        | t     | f    | f     | f   | f
                                                                                              
        import datetime
        now = datetime.datetime.now()
        self.asset_mac  = '%s %s' % (CONFIG.assetTag, CONFIG.mac)
        self.host_type  = '%s %s' % (CONFIG.hostname, CONFIG.hostType)

        segs = CONFIG.assetTag.split("-")
        import commands      
        if len(segs) >3: # this is for UEB #60b099ac-7e7e-42dd-bb96-bda073927077
           UEB_type = "UEB" 
           if len(commands.getoutput("/sbin/lsmod | /bin/grep vmxnet").splitlines()):
              UEB_type = "VMWare"
           elif len(commands.getoutput("/sbin/lsmod | /bin/grep vmbus").splitlines()):  
              UEB_type = "HyperV"
              
           bptool_lines=commands.getoutput("/usr/bp/bin/bptool -v 2>&1| /bin/grep -E 'Feature:'").splitlines()
           UEB_license_type ="N/A"
           if len(bptool_lines):
              theLine =bptool_lines[0]
              #Feature:  ENTRB,MUX=10,VC=INF,RC=INF,D2D=INF,ENC,ADX
              theWord =theLine.split()[1]
              UEB_license_type =theWord.split(",")[0]

           self.host_type +=" %s %s" % (UEB_type, UEB_license_type)   
           
        self.hostname   = CONFIG.hostname
        self.ver_info   = CONFIG.ver_info

        self.category_id = category_id
        self.htime = now
        self.epoch = now.strftime('%s')

        #trap_type_id: 12  is Version Information
        #trap_type_id: 15  is Disk health state has changed
        #trap_type_id: 16  is IPMI Event has occurred
        #trap_type_id: 99  is  Test Trap
        #trap_type_id: 100  is Dynamic Trap to collect platform data
        trap_type_id=category_id
        if category_id==1:
           trap_type_id=12

        self.type       = get_trap_type(trap_type_id)
        self.notification_id = notification_id
        self.username = "root"
        if category_id==15:
            self.username = "diskmon"
            if severity==1:
                self.notification_id =137
            elif severity==2:    
                self.notification_id =138
            else:    
                self.notification_id =139
        elif category_id==16:
            self.username = "ipmi"
            if severity>=3:
                self.notification_id =155
            elif severity==2:    
                self.notification_id =156
            else:
                self.notification_id =157
        elif category_id==20:
            self.username = "dynamic"
            self.book = False    
            CONFIG.bpdb_snmp_config   = []
            CONFIG.bpdb_notifications = []
        else:
           self.notification_id = notification_id
        #elif category_id==1:
        #    self.username = "platform"
        #    self.notification_id = notification_id
        

        self.alert  = False
        self.trap   = False
        self.log    = False
        self.email  = False
        self.psa    = False

        for note in CONFIG.bpdb_notifications:
            if self.notification_id ==note.notification_id:
               self.alert, self.trap, self.log, self.email, self.psa = note.alert, note.trap, note.log, note.email, note.psa

        self.identifier,self.message,self.severity,self.status,self.closed = identifier,message,severity,status,'t'
        if int(status):  # open
           self.closed  = 'f'

        self.rows = {    0: ("OID",                 "enterprises.21865.1.2.100.0.%d" %trap_type_id, "") # 99: Test  40: originated by disk_monitoring script
                         ,  1: ("Type",                "enterprises.21865.1.1.1",        "s")
                         ,  2: ("Identifier",          "enterprises.21865.1.1.2",        "s")
                         ,  3: ("Message",             "enterprises.21865.1.1.3",        "s")
                         ,  4: ("Severity",            "enterprises.21865.1.1.4",        "i") # 1: FATAL  2: WARNING  3: NOTICE  4: VERBOSE
                         ,  5: ("Status",              "enterprises.21865.1.1.5",        "i") # 0: Clear/Close  1: Raise/Open
                         ,  6: ("Asset/MAC",           "enterprises.21865.1.1.6",        "s")
                         ,  7: ("Hostname/Type",       "enterprises.21865.1.1.7",        "s")
                         ,  8: ("Version/InstallDate", "enterprises.21865.1.1.8",        "s")
                         }

        objs = ['']
        objs.append(self.type)
        objs.append(self.identifier)
        objs.append(self.message)
        objs.append(self.severity)
        objs.append(self.status)
        objs.append(self.asset_mac)
        objs.append(self.host_type)
        objs.append(self.ver_info)
        self.objs = objs

    def __repr__(self):
        output = ""
        for i in range(9):
            objOID  = self.rows[i][1]
            if i==0:
                output += "%s%s\n" % (objOID, self.objs[i])
            else:
                output += "%s:= %s\n" % (objOID, self.objs[i])
        return output    


    def does_pass_threshold(self, days):
       """threshold should be passed to send another trap"""
       if not CONFIG.dbON:  # don't want to send a trap if db is down for analysis trap...
          return False
       #if CONFIG.dbON and os.path.exists("/var/log/monitoring_disks/bp.snmp_history.dbout"):
       #   command = "tac /var/log/monitoring_disks/bp.snmp_history.dbout | grep %s" % shortName 
       identifier =self.identifier
       psql_command ="/usr/bin/psql bpdb postgres -tq -c \""
       psql_command+="SELECT id,oid,time_sent,destination,community,traptype,regexp_replace(object,E'[\\n\\r]+',' ','g'),severity,status,description "
       psql_command+="FROM bp.snmp_history WHERE object LIKE '%%%s' ORDER BY id DESC LIMIT 1\" | grep -v ^$" % (identifier)
       import commands      
       import time, datetime
       lines = commands.getoutput(psql_command).splitlines()
       if len(lines):
          words = lines[0].split("|")
          time_sent =self.get_time_stamp(words[2].strip())
          offset = days*24
          threshold = time_sent + datetime.timedelta(hours=offset)
          now = datetime.datetime.now()
          if now>threshold:
             return True
          else:
             return False
          
       return True  # if no entries found, then we have to send a trap

    def get_time_stamp(self, stat_time):
       import datetime
       words = stat_time.split()
       #-rw-rw-r-- 1 root loggers     622 2013-07-30 13:29:12.324820223 -0400 /usr/bp/logs.dir/vcd-worker-5.log
       dateSegs = words[0].split("-")
       timeSegs = words[1].split(":")
       timestamp_start = datetime.datetime(int(dateSegs[0]), int(dateSegs[1]), int(dateSegs[2]), int(timeSegs[0]), int(timeSegs[1]), int(float(timeSegs[2])))
       return timestamp_start

                            
    def send_trap(self, doSendInfoTrap=False):  # log as well
        # community and ipaddress should be from bp.snmp_config table
        isLogged = False

        for snmp_cfg in CONFIG.bpdb_snmp_config:
           
            if not CONFIG.confirm and CONFIG.selfhealing:
               print "[Testing Purpose] NOT confirmed and showing the message::  %s" % self.message
               
            if self.log and not isLogged:
               self.book_to_audit_history()
               isLogged = True
               
            if snmp_cfg.enabled and self.trap and CONFIG.confirm:

               # Do booking
               if CONFIG.dbON:    
                  theMessage = self.message
                  if not (self.severity ==1 or doSendInfoTrap):
                     theMessage += " >>NOTSENT"

                  if CONFIG.verbose:   
                     print "trap.py:: %s" %theMessage  
                  
                  query  = "INSERT INTO bp.snmp_history (oid, destination, community, traptype, object, description, severity, status) "
                  query += "VALUES ('%s','%s','%s','%s','%s','%s',%s,%s)" \
                           % (self.rows[0][1], snmp_cfg.destination, snmp_cfg.community, self.type, self.identifier, theMessage, self.severity, self.status)
                  print query
                  self.run_query(query)
               else:
                  if not os.path.exists("/var/log/monitoring_disks/bp.snmp_history.dbout"):
                     os.system("echo > /var/log/monitoring_disks/bp.snmp_history.dbout")
                     
                  dbentry = " 0 | %s | %s | %s | %s | %s | %s | %s | %s | %s " \
                            % (self.rows[0][1], self.htime, snmp_cfg.destination, snmp_cfg.community, self.type, self.identifier, self.severity, self.status, theMessage)
                  print dbentry
                  os.system("echo '%s' >> /var/log/monitoring_disks/bp.snmp_history.dbout" % dbentry)

               # Do send
               if self.severity == 1 or doSendInfoTrap:  # Sometimes, I don't send a trap but logging into snmp_history
                  command = "/usr/bin/snmptrap -v 2c -c %s %s '' " % (snmp_cfg.community, snmp_cfg.destination)
                  for i in range(9):
                     objOID  = self.rows[i][1]
                     objType = self.rows[i][2] # either string or integer
                     if objType == "s":
                        command += "%s %s '%s' " % (objOID, objType, self.objs[i])
                     else:
                        command += "%s %s %s "   % (objOID, objType, self.objs[i])
                        
                  if CONFIG.verbose:
                     print "trap.py:: %s" %command
                  if not CONFIG.debug:
                     os.system(command)

                                                            

    def book_to_alerts(self): # psa as well   
       #if self.test: return # this is designed for trap only 
        
       if self.alert:
          query  = "INSERT INTO bp.alerts (alert_text, source_id, target, severity, created, updated, closed, notification_id) "
          query += "VALUES ('%s',%d,'%s',%s,%s,%s,'%s',%d)" % (self.message, self.category_id, self.identifier, self.severity, self.epoch, self.epoch, self.closed, self.notification_id)
          if CONFIG.confirm and self.severity==1 :
             self.run_query(query) # not attempted if dbON=False

       if len(CONFIG.psaON) and self.severity==1 and self.psa:
          if not CONFIG.debug and CONFIG.confirm:
             os.system("/usr/bp/bin/cmc_psa %s %d '%s'" % (CONFIG.psaON, self.notification_id, self.message))
       return

    def book_to_audit_history(self):
        query  = "INSERT INTO bp.audit_history (notification_id, event_time, username, system, message) "
        query += "VALUES (%d, '%s','%s','%s','%s')" % (self.notification_id, self.htime, self.username, self.hostname, self.message)

        # To avoid flooding insert calls every 30 minutes when database is locked
        import commands
        command ="/bin/ps aux | /bin/grep 'INSERT INTO bp.audit_history' | /bin/grep diskmon"
        if not len(commands.getoutput(command).splitlines()):
           if CONFIG.confirm:
              self.run_query(query)
        return

    def run_command(self, command, secondsToSleep=0):
        message = "Executing # %s " % command
        if CONFIG.confirm:
            #print message
            os.system(command)
            if secondsToSleep>0:
                import time
                time.sleep(secondsToSleep)
        else:
            message = message + " (NOT EXECUTED)"
            print message
   
        return

    def run_query(self, query):
        command = "/usr/bin/psql bpdb postgres -c \"%s\" >/dev/null 2>&1" % query
        if CONFIG.dbON:
            self.run_command(command)
        return

