#!/usr/bin/env python
from M2Crypto import RSA
import sha
algo = 'sha1'


import logging
logger = logging.getLogger(__name__)
logger.setLevel(logging.DEBUG)

# This is required in python version < 2.6 to surpress log messages for unittests
class NullHandler(logging.Handler):
	def emit(self, record):
		pass
handler = NullHandler()
logger.addHandler(handler)

def verify(input_file_path, public_key_path, signature_file_path = ""):
    """
    Verifies the signature contained in the file

    Keyword Parameters:
    string input_file_path - path to the input file
    string public_key_path - path to the public key file
    string signature_file_path - path to the signature file

    Returns:
    bool - True if it verifies; False if it does not; None if there was an error
    """
    try:
        message = open(input_file_path, 'r').read()
    except:
        logger.warning( "the input file could not be loaded for reading")
        return None
    digest = sha.sha(message).digest()

    try:
        rsa_public = RSA.load_pub_key(public_key_path)
    except:
        logger.warning( "the public key could not be loaded")
        return None
    if signature_file_path == "":
        signature_file_path = input_file_path+".sig"
    try:
        signature = open(signature_file_path,'r').read()
    except:
        logger.warning("the signature file could not be open")
        return None

    try:
        verify = rsa_public.verify(digest, signature, algo)
    except:
        return False
    return verify==1
